@@ -1,7 +1,7 @@
 #!/usr/bin/env python3
 # build.py --- Convenience script for building and running Jami
 
-# Copyright (C) 2016-2023 Savoir-faire Linux Inc.
+# Copyright (C) 2016-2025 Savoir-faire Linux Inc.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -45,7 +45,7 @@ APT_BASED_DISTROS = [
 ]
 
 DNF_BASED_DISTROS = [
-    'fedora', 'rhel',
+    'fedora', 'rhel', 'almalinux',
 ]
 
 PACMAN_BASED_DISTROS = [
@@ -99,8 +99,8 @@ ZYPPER_DEPENDENCIES = [
     # daemon
     'speexdsp-devel', 'speex-devel', 'libdbus-c++-devel', 'jsoncpp-devel', 'yaml-cpp-devel',
     'yasm', 'libuuid-devel', 'libnettle-devel', 'libopus-devel', 'libexpat-devel',
-    'libgnutls-devel', 'msgpack-devel', 'libavcodec-devel', 'libavdevice-devel', 'pcre-devel',
-    'alsa-devel', 'libpulse-devel', 'libudev-devel', 'libva-devel', 'libvdpau-devel',
+    'libgnutls-devel', 'msgpack-c-devel', 'msgpack-cxx-devel', 'libavcodec-devel', 'libavdevice-devel', 'pcre-devel',
+    'alsa-devel', 'libpulse-devel', 'libudev-devel', 'libva-devel', 'libvdpau-devel', 'pipewire-devel',
     'libopenssl-devel', 'libavutil-devel',
 ]
 
@@ -108,9 +108,11 @@ ZYPPER_CLIENT_DEPENDENCIES = [
     # lrc
     'qt6-core-devel', 'qt6-dbus-devel', 'qt6-linguist-devel',
     # client-qt
-    'qt6-svg-devel', 'qt6-multimedia-devel', 'qt6-declarative-devel',
-    'qt6-quickcontrols2-devel',
-    'qrencode-devel', 'NetworkManager-devel'
+    'qt6-concurrent-devel', 'qt6-qt5compat-devel', 'qt6-qt5compat-imports',
+    'qt6-svg-devel', 'qt6-multimedia-devel', 'qt6-multimedia-imports',
+    'qt6-declarative-devel', 'qt6-qmlcompiler-private-devel',
+    'qt6-quickcontrols2-devel', 'qt6-shadertools-devel',
+    'qrencode-devel', 'NetworkManager-devel', 'hunspell-devel', 'libhunspell-devel'
 ]
 
 ZYPPER_QT_WEBENGINE = [
@@ -128,7 +130,7 @@ DNF_DEPENDENCIES = [
     'gcc-c++', 'which', 'alsa-lib-devel', 'systemd-devel', 'libuuid-devel',
     'uuid-devel', 'gnutls-devel', 'nettle-devel', 'opus-devel', 'speexdsp-devel',
     'yaml-cpp-devel', 'swig', 'jsoncpp-devel',
-    'patch', 'libva-devel', 'openssl-devel', 'libvdpau-devel', 'msgpack-devel',
+    'patch', 'libva-devel', 'openssl-devel', 'libvdpau-devel', 'pipewire-devel', 'msgpack-devel',
     'sqlite-devel', 'openssl-static', 'pandoc', 'nasm',
     'bzip2'
 ]
@@ -137,7 +139,7 @@ DNF_CLIENT_DEPENDENCIES = [
     'libnotify-devel',
     'qt6-qtbase-devel',
     'qt6-qtsvg-devel', 'qt6-qtmultimedia-devel', 'qt6-qtdeclarative-devel',
-    'qrencode-devel', 'NetworkManager-libnm-devel'
+    'qrencode-devel', 'NetworkManager-libnm-devel', 'hunspell-devel', 'libhunspell-devel'
 ]
 
 DNF_QT_WEBENGINE = ['qt6-qtwebengine-devel']
@@ -152,7 +154,7 @@ APT_DEPENDENCIES = [
     'libopus-dev', 'libpcre3-dev', 'libpulse-dev', 'libssl-dev',
     'libspeex-dev', 'libspeexdsp-dev', 'libswscale-dev', 'libtool',
     'libudev-dev', 'libyaml-cpp-dev', 'sip-tester', 'swig',
-    'uuid-dev', 'yasm', 'libjsoncpp-dev', 'libva-dev', 'libvdpau-dev', 'libmsgpack-dev',
+    'uuid-dev', 'yasm', 'libjsoncpp-dev', 'libva-dev', 'libvdpau-dev', 'libpipewire-0.3-dev', 'libmsgpack-dev',
     'pandoc', 'nasm', 'dpkg-dev', 'libsystemd-dev'
 ]
 
@@ -169,7 +171,7 @@ APT_CLIENT_DEPENDENCIES = [
     'qml6-module-qtquick-dialogs', 'qml6-module-qtquick-layouts',
     'qml6-module-qtquick-shapes', 'qml6-module-qtquick-window',
     'qml6-module-qtquick-templates', 'qml6-module-qt-labs-platform',
-    'libqrencode-dev', 'libnm-dev'
+    'libqrencode-dev', 'libnm-dev', 'hunspell', 'libhunspell-dev'
 ]
 
 APT_QT_WEBENGINE = [
@@ -192,7 +194,7 @@ PACMAN_CLIENT_DEPENDENCIES = [
     'qt6-declarative', 'qt6-5compat', 'qt6-multimedia',
     'qt6-networkauth', 'qt6-shadertools',
     'qt6-svg', 'qt6-tools',
-    'qrencode', 'libnm'
+    'qrencode', 'libnm', 'hunspell'
 ]
 
 PACMAN_QT_WEBENGINE = ['qt6-webengine']
@@ -244,8 +246,8 @@ def run_dependencies(args):
 
     elif args.distribution in DNF_BASED_DISTROS:
         if args.assume_yes:
-            for i, _ in enumerate(DNF_INSTALL_SCRIPT):
-                DNF_INSTALL_SCRIPT[i] += ASSUME_YES_FLAG
+            for i, _ in enumerate(RPM_INSTALL_SCRIPT):
+                RPM_INSTALL_SCRIPT[i] += ASSUME_YES_FLAG
         execute_script(
             RPM_INSTALL_SCRIPT,
             {"packages": ' '.join(map(shlex.quote, DNF_DEPENDENCIES))})
@@ -297,7 +299,7 @@ def run_dependencies(args):
         print("The win32 version does not install dependencies with this script.\nPlease continue with the --install instruction.")
         sys.exit(1)
     elif args.distribution == 'guix':
-        print(f"Building the profile defined in '{GUIX_MANIFEST}'...")
+        print(f"Building the profile defined in '{GUIX_MANIFEST}'…")
         execute_script([f'guix shell --manifest={GUIX_MANIFEST} -- true'])
 
     else:
@@ -314,7 +316,7 @@ def run_init(args):
     client_hooks_dir = '.git/hooks'
     daemon_hooks_dir = '.git/modules/daemon/hooks'
 
-    print("Installing commit-msg hooks...")
+    print("Installing commit-msg hooks…")
     # Copy the commit-msg hook to all modules in the same way.
     for hooks_dir in [client_hooks_dir, daemon_hooks_dir]:
         if not os.path.exists(hooks_dir):
@@ -322,7 +324,7 @@ def run_init(args):
         copy_file("./extras/scripts/commit-msg",
                   os.path.join(hooks_dir, "commit-msg"))
 
-    print("Installing pre-commit hooks...")
+    print("Installing pre-commit hooks…")
     format_script = "./extras/scripts/format.py"
     # Prepend with the python executable if on Windows (not WSL).
     if sys.platform == 'win32':
@@ -364,13 +366,27 @@ def cwd(path):
 def run_install(args):
     # Platforms with special compilation scripts
     if args.distribution == WIN32_DISTRIBUTION_NAME:
+        # Build daemon if not using pywinmake
         if not args.pywinmake:
             with cwd('daemon/compat/msvc'):
                 execute_script([f'python winmake.py -iv -s {args.sdk} -b daemon'])
 
+        # Prepare the build-windows.py script call
         build_windows = 'extras/scripts/build-windows.py'
-        execute_script([f'python {build_windows} --init'])
-        execute_script([f'python {build_windows} --qt={args.qt}'])
+        # Initialize build environment
+        execute_script([f'python {build_windows} --init --qt={args.qt}'])
+
+        # Construct build command with options
+        build_cmd = [
+            'python',
+            build_windows,
+            f'--qt={args.qt}'
+        ]
+
+        if args.enable_crash_reports:
+            build_cmd.append('--enable-crash-reports')
+
+        execute_script([' '.join(build_cmd)])
         return True
 
     # Unix-like platforms
@@ -387,6 +403,8 @@ def run_install(args):
         install_args.append('-u')
     if args.debug:
         install_args.append('-d')
+    if args.testing:
+        install_args.append('-t')
     if args.asan:
         install_args.append('-A')
     if args.no_libwrap:
@@ -395,6 +413,12 @@ def run_install(args):
         install_args.append('-w')
     if args.arch:
         install_args += ('-a', args.arch)
+    if args.extra_cmake_flags:
+        install_args += ('-D', args.extra_cmake_flags)
+    if args.enable_crash_reports:
+        install_args.append('-C')
+    if args.ignore_system_libs:
+        install_args.append('-i')
 
     if args.distribution == OSX_DISTRIBUTION_NAME:
         # The `universal_newlines` parameter has been renamed to `text` in
@@ -598,7 +622,7 @@ def run_run(args):
             client_process.wait()
 
     except KeyboardInterrupt:
-        print("\nCaught KeyboardInterrupt...")
+        print("\nCaught KeyboardInterrupt…")
 
     finally:
         if args.debug:
@@ -608,7 +632,7 @@ def run_run(args):
             try:
                 # Only kill the processes if they are running, as they
                 # could have been closed by the user.
-                print("Killing processes...")
+                print("Killing processes…")
                 if args.no_libwrap:
                     jamid_log.close()
                     if jamid_process.poll() is None:
@@ -723,6 +747,9 @@ def parse_args():
                     default=True, action='store_false')
     ap.add_argument('--qt', type=str,
                     help='Use the Qt path supplied')
+    ap.add_argument('--testing', dest='testing',
+                    default=False, action='store_true',
+                    help='Enable testing for both client and daemon')
     ap.add_argument('--no-libwrap', dest='no_libwrap',
                     default=False, action='store_true',
                     help='Disable libwrap. Also set --disable-shared option to daemon configure')
@@ -738,6 +765,15 @@ def parse_args():
     ap.add_argument('--pywinmake', dest='pywinmake',
                     default=False, action='store_true',
                     help='Build Jami for Windows using pywinmake')
+    # Allow supplying extra congifure flags to the client cmake.
+    ap.add_argument('--extra-cmake-flags', type=str,
+                    help='Extra flags to pass to the client cmake')
+    ap.add_argument('--enable-crash-reports',
+                    action='store_true', default=False,
+                    help='Enable crash reporting')
+    ap.add_argument('--ignore_system_libs', dest='ignore_system_libs',
+                    default=False, action='store_true',
+                    help='Build contrib packages regardless of system detection')
 
     dist = choose_distribution()
 
