@@ -1,25 +1,18 @@
 /*
- *  Copyright (C) 2004-2023 Savoir-faire Linux Inc.
+ *  Copyright (C) 2004-2025 Savoir-faire Linux Inc.
  *
- *  Author: Pierre-Luc Beaudoin <pierre-luc.beaudoin@savoirfairelinux.com>
- *  Author: Emmanuel Milou <emmanuel.milou@savoirfairelinux.com>
- *  Author: Guillaume Carmel-Archambault <guillaume.carmel-archambault@savoirfairelinux.com>
- *  Author: Guillaume Roguez <Guillaume.Roguez@savoirfairelinux.com>
- *  Author: Philippe Gorley <philippe.gorley@savoirfairelinux.com>
- *
- *  This program is free software; you can redistribute it and/or modify
+ *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 3 of the License, or
+ *  the Free Software Foundation, either version 3 of the License, or
  *  (at your option) any later version.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA.
+ *  along with this program. If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -29,12 +22,9 @@
 #include "configurationmanager_interface.h"
 #include "account_schema.h"
 #include "manager.h"
-#include "connectivity/security/tlsvalidator.h"
-#include "connectivity/security/certstore.h"
 #include "logger.h"
 #include "fileutils.h"
 #include "archiver.h"
-#include "connectivity/ip_utils.h"
 #include "sip/sipaccount.h"
 #include "jamidht/jamiaccount.h"
 #include "sip/sipaccount_config.h"
@@ -43,8 +33,14 @@
 #include "system_codec_container.h"
 #include "account_const.h"
 #include "client/ring_signal.h"
-#include "connectivity/upnp/upnp_context.h"
 #include "audio/ringbufferpool.h"
+#include "connectivity/security/tlsvalidator.h"
+
+#include <dhtnet/ip_utils.h>
+#include <dhtnet/upnp/upnp_context.h>
+#include <dhtnet/certstore.h>
+
+#include <regex>
 
 #ifdef __APPLE__
 #include <TargetConditionals.h>
@@ -64,6 +60,8 @@
 #undef interface
 #endif
 
+#include <string_view>
+
 namespace libjami {
 
 constexpr unsigned CODECS_NOT_LOADED = 0x1000; /** Codecs not found */
@@ -96,7 +94,8 @@ validateCertificate(const std::string& accountId, const std::string& certificate
 {
     try {
         if (const auto acc = jami::Manager::instance().getAccount<jami::JamiAccount>(accountId))
-            return TlsValidator {acc->certStore(), acc->certStore().getCertificate(certificate)}.getSerializedChecks();
+            return TlsValidator {acc->certStore(), acc->certStore().getCertificate(certificate)}
+                .getSerializedChecks();
     } catch (const std::runtime_error& e) {
         JAMI_WARN("Certificate loading failed: %s", e.what());
     }
@@ -112,7 +111,8 @@ validateCertificatePath(const std::string& accountId,
 {
     try {
         if (const auto acc = jami::Manager::instance().getAccount<jami::JamiAccount>(accountId))
-            return TlsValidator {acc->certStore(), certificate, privateKey, privateKeyPass, caList}.getSerializedChecks();
+            return TlsValidator {acc->certStore(), certificate, privateKey, privateKeyPass, caList}
+                .getSerializedChecks();
     } catch (const std::runtime_error& e) {
         JAMI_WARN("Certificate loading failed: %s", e.what());
         return {{Certificate::ChecksNames::EXIST, Certificate::CheckValuesNames::FAILED}};
@@ -125,7 +125,8 @@ getCertificateDetails(const std::string& accountId, const std::string& certifica
 {
     try {
         if (const auto acc = jami::Manager::instance().getAccount<jami::JamiAccount>(accountId))
-            return TlsValidator {acc->certStore(), acc->certStore().getCertificate(certificate)}.getSerializedDetails();
+            return TlsValidator {acc->certStore(), acc->certStore().getCertificate(certificate)}
+                .getSerializedDetails();
     } catch (const std::runtime_error& e) {
         JAMI_WARN("Certificate loading failed: %s", e.what());
     }
@@ -209,7 +210,7 @@ setCertificateStatus(const std::string& accountId,
 {
     try {
         if (auto acc = jami::Manager::instance().getAccount<jami::JamiAccount>(accountId)) {
-            auto status = jami::tls::TrustStore::statusFromStr(ststr.c_str());
+            auto status = dhtnet::tls::TrustStore::statusFromStr(ststr.c_str());
             return acc->setCertificateStatus(certId, status);
         }
     } catch (const std::out_of_range&) {
@@ -220,7 +221,7 @@ setCertificateStatus(const std::string& accountId,
 std::vector<std::string>
 getCertificatesByStatus(const std::string& accountId, const std::string& ststr)
 {
-    auto status = jami::tls::TrustStore::statusFromStr(ststr.c_str());
+    auto status = dhtnet::tls::TrustStore::statusFromStr(ststr.c_str());
     if (auto acc = jami::Manager::instance().getAccount<jami::JamiAccount>(accountId))
         return acc->getCertificatesByStatus(status);
     return {};
@@ -238,6 +239,12 @@ setAccountActive(const std::string& accountId, bool enable, bool shutdownConnect
     jami::Manager::instance().setAccountActive(accountId, enable, shutdownConnections);
 }
 
+void
+loadAccountAndConversation(const std::string& accountId, bool loadAll, const std::string& convId)
+{
+    jami::Manager::instance().loadAccountAndConversation(accountId, loadAll, convId);
+}
+
 void
 sendRegister(const std::string& accountId, bool enable)
 {
@@ -247,7 +254,17 @@ sendRegister(const std::string& accountId, bool enable)
 bool
 isPasswordValid(const std::string& accountId, const std::string& password)
 {
-    return jami::Manager::instance().isPasswordValid(accountId, password);
+    if (auto acc = jami::Manager::instance().getAccount<JamiAccount>(accountId))
+        return acc->isPasswordValid(password);
+    return false;
+}
+
+std::vector<uint8_t>
+getPasswordKey(const std::string& accountID, const std::string& password)
+{
+    if (auto acc = jami::Manager::instance().getAccount<JamiAccount>(accountID))
+        return acc->getPasswordKey(password);
+    return {};
 }
 
 void
@@ -280,6 +297,18 @@ getNearbyPeers(const std::string& accountId)
     return jami::Manager::instance().getNearbyPeers(accountId);
 }
 
+void
+updateProfile(const std::string& accountId,
+              const std::string& displayName,
+              const std::string& avatar,
+              const std::string& fileType,
+              int32_t flag)
+{
+    if (const auto acc = jami::Manager::instance().getAccount(accountId)) {
+        acc->updateProfile(displayName, avatar, fileType, flag);
+    }
+}
+
 int
 getMessageStatus(uint64_t messageId)
 {
@@ -295,8 +324,6 @@ getMessageStatus(const std::string& accountId, uint64_t messageId)
 bool
 cancelMessage(const std::string& accountId, uint64_t messageId)
 {
-    if (const auto acc = jami::Manager::instance().getAccount(accountId))
-        return acc->cancelMessage(messageId);
     return {};
 }
 
@@ -318,11 +345,42 @@ setMessageDisplayed(const std::string& accountId,
     return false;
 }
 
+int32_t
+addDevice(const std::string& accountId, const std::string& uri)
+{
+    JAMI_DEBUG("[LinkDevice {}] exportToPeer called.", accountId);
+    if (const auto account = jami::Manager::instance().getAccount<jami::JamiAccount>(accountId)) {
+        return account->addDevice(uri);
+    }
+    return static_cast<int32_t>(jami::AccountManager::AddDeviceError::GENERIC);
+}
+
 bool
-exportOnRing(const std::string& accountId, const std::string& password)
+confirmAddDevice(const std::string& accountId, uint32_t op_id)
 {
     if (const auto account = jami::Manager::instance().getAccount<jami::JamiAccount>(accountId)) {
-        account->addDevice(password);
+        return account->confirmAddDevice(op_id);
+    }
+    return false;
+}
+
+bool
+cancelAddDevice(const std::string& accountId, uint32_t op_id)
+{
+    if (const auto account = jami::Manager::instance().getAccount<jami::JamiAccount>(accountId)) {
+        return account->cancelAddDevice(op_id);
+    }
+    return false;
+}
+
+bool
+provideAccountAuthentication(const std::string& accountId,
+                             const std::string& credentialsFromUser,
+                             const std::string& scheme)
+{
+    if (const auto account = jami::Manager::instance().getAccount<jami::JamiAccount>(accountId)) {
+        // send the password to the channel for communicationg with the old device
+        account->provideAccountAuthentication(credentialsFromUser, scheme);
         return true;
     }
     return false;
@@ -331,19 +389,23 @@ exportOnRing(const std::string& accountId, const std::string& password)
 bool
 exportToFile(const std::string& accountId,
              const std::string& destinationPath,
+             const std::string& scheme,
              const std::string& password)
 {
     if (const auto account = jami::Manager::instance().getAccount<jami::JamiAccount>(accountId)) {
-        return account->exportArchive(destinationPath, password);
+        return account->exportArchive(destinationPath, scheme, password);
     }
     return false;
 }
 
 bool
-revokeDevice(const std::string& accountId, const std::string& password, const std::string& deviceID)
+revokeDevice(const std::string& accountId,
+             const std::string& deviceId,
+             const std::string& scheme,
+             const std::string& password)
 {
     if (const auto account = jami::Manager::instance().getAccount<jami::JamiAccount>(accountId)) {
-        return account->revokeDevice(password, deviceID);
+        return account->revokeDevice(deviceId, scheme, password);
     }
     return false;
 }
@@ -455,6 +517,18 @@ monitor(bool continuous)
     return jami::Manager::instance().monitor(continuous);
 }
 
+std::vector<std::map<std::string, std::string>>
+getConnectionList(const std::string& accountId, const std::string& conversationId)
+{
+    return jami::Manager::instance().getConnectionList(accountId, conversationId);
+}
+
+std::vector<std::map<std::string, std::string>>
+getChannelList(const std::string& accountId, const std::string& connectionId)
+{
+    return jami::Manager::instance().getChannelList(accountId, connectionId);
+}
+
 void
 removeAccount(const std::string& accountId)
 {
@@ -503,32 +577,31 @@ setCodecDetails(const std::string& accountId,
 {
     auto acc = jami::Manager::instance().getAccount(accountId);
     if (!acc) {
-        JAMI_ERR("Could not find account %s. can not set codec details", accountId.c_str());
+        JAMI_ERR("Unable to find account %s. Unable to set codec details", accountId.c_str());
         return false;
     }
 
     auto codec = acc->searchCodecById(codecId, jami::MEDIA_ALL);
     if (!codec) {
-        JAMI_ERR("can not find codec %d", codecId);
+        JAMI_ERR("Unable to find codec %d", codecId);
         return false;
     }
     try {
-        if (codec->systemCodecInfo.mediaType & jami::MEDIA_AUDIO) {
-            if (auto foundCodec = std::static_pointer_cast<jami::AccountAudioCodecInfo>(codec)) {
+        if (codec->mediaType & jami::MEDIA_AUDIO) {
+            if (auto foundCodec = std::static_pointer_cast<jami::SystemAudioCodecInfo>(codec)) {
                 foundCodec->setCodecSpecifications(details);
                 jami::emitSignal<ConfigurationSignal::MediaParametersChanged>(accountId);
                 return true;
             }
         }
 
-        if (codec->systemCodecInfo.mediaType & jami::MEDIA_VIDEO) {
-            if (auto foundCodec = std::static_pointer_cast<jami::AccountVideoCodecInfo>(codec)) {
+        if (codec->mediaType & jami::MEDIA_VIDEO) {
+            if (auto foundCodec = std::static_pointer_cast<jami::SystemVideoCodecInfo>(codec)) {
                 foundCodec->setCodecSpecifications(details);
-                JAMI_WARN("parameters for %s changed ", foundCodec->systemCodecInfo.name.c_str());
+                JAMI_WARN("parameters for %s changed ", foundCodec->name.c_str());
                 if (auto call = jami::Manager::instance().getCurrentCall()) {
                     if (call->getVideoCodec() == foundCodec) {
-                        JAMI_WARN("%s running. Need to restart encoding",
-                                  foundCodec->systemCodecInfo.name.c_str());
+                        JAMI_WARN("%s running. Need to restart encoding", foundCodec->name.c_str());
                         call->restartMediaSender();
                     }
                 }
@@ -537,7 +610,7 @@ setCodecDetails(const std::string& accountId,
             }
         }
     } catch (const std::exception& e) {
-        JAMI_ERR("Cannot set codec specifications: %s", e.what());
+        JAMI_ERR("Unable to set codec specifications: %s", e.what());
     }
 
     return false;
@@ -548,7 +621,7 @@ getCodecDetails(const std::string& accountId, const unsigned& codecId)
 {
     auto acc = jami::Manager::instance().getAccount(accountId);
     if (!acc) {
-        JAMI_ERR("Could not find account %s return default codec details", accountId.c_str());
+        JAMI_ERR("Unable to find account %s return default codec details", accountId.c_str());
         return jami::Account::getDefaultCodecDetails(codecId);
     }
 
@@ -558,12 +631,12 @@ getCodecDetails(const std::string& accountId, const unsigned& codecId)
         return {};
     }
 
-    if (codec->systemCodecInfo.mediaType & jami::MEDIA_AUDIO)
-        if (auto foundCodec = std::static_pointer_cast<jami::AccountAudioCodecInfo>(codec))
+    if (codec->mediaType & jami::MEDIA_AUDIO)
+        if (auto foundCodec = std::static_pointer_cast<jami::SystemAudioCodecInfo>(codec))
             return foundCodec->getCodecSpecifications();
 
-    if (codec->systemCodecInfo.mediaType & jami::MEDIA_VIDEO)
-        if (auto foundCodec = std::static_pointer_cast<jami::AccountVideoCodecInfo>(codec))
+    if (codec->mediaType & jami::MEDIA_VIDEO)
+        if (auto foundCodec = std::static_pointer_cast<jami::SystemVideoCodecInfo>(codec))
             return foundCodec->getCodecSpecifications();
 
     jami::emitSignal<ConfigurationSignal::Error>(CODECS_NOT_LOADED);
@@ -575,7 +648,7 @@ getActiveCodecList(const std::string& accountId)
 {
     if (auto acc = jami::Manager::instance().getAccount(accountId))
         return acc->getActiveCodecs();
-    JAMI_ERR("Could not find account %s, returning default", accountId.c_str());
+    JAMI_ERR("Unable to find account %s, returning default", accountId.c_str());
     return jami::Account::getDefaultCodecsId();
 }
 
@@ -586,7 +659,7 @@ setActiveCodecList(const std::string& accountId, const std::vector<unsigned>& li
         acc->setActiveCodecs(list);
         jami::Manager::instance().saveConfig(acc);
     } else {
-        JAMI_ERR("Could not find account %s", accountId.c_str());
+        JAMI_ERR("Unable to find account %s", accountId.c_str());
     }
 }
 
@@ -906,19 +979,19 @@ setAccountsOrder(const std::string& order)
 std::string
 getAddrFromInterfaceName(const std::string& interface)
 {
-    return jami::ip_utils::getInterfaceAddr(interface, AF_INET);
+    return dhtnet::ip_utils::getInterfaceAddr(interface, AF_INET);
 }
 
 std::vector<std::string>
 getAllIpInterface()
 {
-    return jami::ip_utils::getAllIpInterface();
+    return dhtnet::ip_utils::getAllIpInterface();
 }
 
 std::vector<std::string>
 getAllIpInterfaceByName()
 {
-    return jami::ip_utils::getAllIpInterfaceByName();
+    return dhtnet::ip_utils::getAllIpInterfaceByName();
 }
 
 std::vector<std::map<std::string, std::string>>
@@ -934,13 +1007,12 @@ setCredentials(const std::string& accountId,
                const std::vector<std::map<std::string, std::string>>& details)
 {
     if (auto sipaccount = jami::Manager::instance().getAccount<SIPAccount>(accountId)) {
-        sipaccount->doUnregister([&](bool /* transport_free */) {
-            sipaccount->editConfig(
-                [&](jami::SipAccountConfig& config) { config.setCredentials(details); });
-            sipaccount->loadConfig();
-            if (sipaccount->isEnabled())
-                sipaccount->doRegister();
-        });
+        sipaccount->doUnregister();
+        sipaccount->editConfig(
+            [&](jami::SipAccountConfig& config) { config.setCredentials(details); });
+        sipaccount->loadConfig();
+        if (sipaccount->isEnabled())
+            sipaccount->doRegister();
         jami::Manager::instance().saveConfig(sipaccount);
     }
 }
@@ -948,12 +1020,12 @@ setCredentials(const std::string& accountId,
 void
 connectivityChanged()
 {
-    JAMI_WARN("received connectivity changed - trying to re-connect enabled accounts");
+    JAMI_WARN("received connectivity changed - attempting to re-connect enabled accounts");
 
     // reset the UPnP context
 #if !(defined(TARGET_OS_IOS) && TARGET_OS_IOS)
     try {
-        jami::upnp::UPnPContext::getUPnPContext()->connectivityChanged();
+        jami::Manager::instance().upnpContext()->connectivityChanged();
     } catch (std::runtime_error& e) {
         JAMI_ERR("UPnP context error: %s", e.what());
     }
@@ -967,13 +1039,16 @@ connectivityChanged()
 bool
 lookupName(const std::string& account, const std::string& nameserver, const std::string& name)
 {
-#if HAVE_RINGNS
+#ifdef ENABLE_NAMESERVER
     if (account.empty()) {
-        auto cb = [name](const std::string& result, jami::NameDirectory::Response response) {
+        auto cb = [name](const std::string& regName,
+                         const std::string& address,
+                         jami::NameDirectory::Response response) {
             jami::emitSignal<libjami::ConfigurationSignal::RegisteredNameFound>("",
+                                                                                name,
                                                                                 (int) response,
-                                                                                result,
-                                                                                name);
+                                                                                address,
+                                                                                regName);
         };
         if (nameserver.empty())
             jami::NameDirectory::lookupUri(name, "", cb);
@@ -991,14 +1066,15 @@ lookupName(const std::string& account, const std::string& nameserver, const std:
 bool
 lookupAddress(const std::string& account, const std::string& nameserver, const std::string& address)
 {
-#if HAVE_RINGNS
+#ifdef ENABLE_NAMESERVER
     if (account.empty()) {
         jami::NameDirectory::instance(nameserver)
             .lookupAddress(address,
-                           [address](const std::string& result,
+                           [address](const std::string& regName,
+                                     const std::string& addr,
                                      jami::NameDirectory::Response response) {
                                jami::emitSignal<libjami::ConfigurationSignal::RegisteredNameFound>(
-                                   "", (int) response, address, result);
+                                   "", address, (int) response, addr, regName);
                            });
         return true;
     } else if (auto acc = jami::Manager::instance().getAccount<JamiAccount>(account)) {
@@ -1019,11 +1095,14 @@ searchUser(const std::string& account, const std::string& query)
 }
 
 bool
-registerName(const std::string& account, const std::string& password, const std::string& name)
+registerName(const std::string& account,
+             const std::string& name,
+             const std::string& scheme,
+             const std::string& password)
 {
-#if HAVE_RINGNS
+#ifdef ENABLE_NAMESERVER
     if (auto acc = jami::Manager::instance().getAccount<JamiAccount>(account)) {
-        acc->registerName(password, name);
+        acc->registerName(name, scheme, password);
         return true;
     }
 #endif
@@ -1123,4 +1202,10 @@ isAllModerators(const std::string& accountId)
     return jami::Manager::instance().isAllModerators(accountId);
 }
 
+void
+setResourceDirPath(const std::string& resourceDir)
+{
+    jami::fileutils::set_resource_dir_path(resourceDir);
+}
+
 } // namespace libjami
