@@ -1,30 +1,19 @@
 /*
- *  Copyright (C) 2004-2023 Savoir-faire Linux Inc.
+ *  Copyright (C) 2004-2025 Savoir-faire Linux Inc.
  *
- *  Author: Yan Morin <yan.morin@savoirfairelinux.com>
- *  Author: Laurielle Lea <laurielle.lea@savoirfairelinux.com>
- *  Author: Emmanuel Milou <emmanuel.milou@savoirfairelinux.com>
- *  Author: Guillaume Carmel-Archambault <guillaume.carmel-archambault@savoirfairelinux.com>
- *  Author: Alexandre Savard <alexandre.savard@savoirfairelinux.com>
- *  Author: Tristan Matthews <tristan.matthews@savoirfairelinux.com>
- *  Author: Guillaume Roguez <guillaume.roguez@savoirfairelinux.com>
- *  Author: Aline Gondim Santos <aline.gondimsantos@savoirfairelinux.com>
- *
- *  This program is free software; you can redistribute it and/or modify
+ *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 3 of the License, or
+ *  the Free Software Foundation, either version 3 of the License, or
  *  (at your option) any later version.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA.
+ *  along with this program. If not, see <https://www.gnu.org/licenses/>.
  */
-
 #pragma once
 
 #ifdef HAVE_CONFIG_H
@@ -34,10 +23,10 @@
 #include "account_factory.h"
 #include "call_factory.h"
 #include "preferences.h"
-#include "audio/audiolayer.h"
+#include "media/audio/audiolayer.h"
 #include "scheduled_executor.h"
 #include "gittransport.h"
-#include "connectivity/security/certstore.h"
+#include <dhtnet/certstore.h>
 
 #include <algorithm>
 #include <atomic>
@@ -54,31 +43,34 @@ namespace asio {
 class io_context;
 }
 
+namespace dhtnet {
+class ChannelSocket;
+class IceTransportFactory;
+} // namespace dhtnet
+
 namespace jami {
 namespace video {
 class SinkClient;
 class VideoGenerator;
 } // namespace video
-class ChannelSocket;
 class RingBufferPool;
 struct VideoManager;
 class Conference;
 class AudioLoop;
-class IceTransportFactory;
 class JamiAccount;
 class SIPVoIPLink;
 class JamiPluginManager;
 
 /** Manager (controller) of daemon */
 // TODO LIBJAMI_PUBLIC only if tests
-class LIBJAMI_TESTABLE Manager
+class LIBJAMI_TEST_EXPORT Manager
 {
 private:
     std::mt19937_64 rand_;
 
 public:
     // TODO LIBJAMI_PUBLIC only if tests
-    static LIBJAMI_TESTABLE Manager& instance();
+    static LIBJAMI_TEST_EXPORT Manager& instance();
 
     void setAutoAnswer(bool enable);
 
@@ -121,11 +113,13 @@ public:
 
     static bool syncOnRegister;
 
+    static bool autoLoad;
+
     /**
      * Initialisation of thread (sound) and map.
-     * Init a new VoIPLink, audio codec and audio driver
+     * Init a new VoIPLink, audio codec and audio driver.
      */
-    void init(const std::string& config_file, libjami::InitFlag flags);
+    void init(const std::filesystem::path& config_file, libjami::InitFlag flags);
 
     /*
      * Terminate all threads and exit DBus loop
@@ -134,6 +128,11 @@ public:
 
     void monitor(bool continuous);
 
+    std::vector<std::map<std::string, std::string>> getConnectionList(
+        const std::string& accountId, const std::string& conversationId);
+    std::vector<std::map<std::string, std::string>> getChannelList(const std::string& accountId,
+                                                                   const std::string& connectionId);
+
     /**
      * Accessor to audiodriver.
      * it's multi-thread and use mutex internally
@@ -151,7 +150,7 @@ public:
      * @param accountId the user's account ID
      * @param callee the callee's ID/URI. Depends on the account type.
      * Refer to placeCall/placeCallWithMedia documentations.
-     * @param mediaList a list of medias to include
+     * @param mediaList a list of media to include
      * @return the call ID on success, empty string otherwise
      */
     std::string outgoingCall(const std::string& accountId,
@@ -163,10 +162,10 @@ public:
      * Answer the call
      * @param callId
      */
-    bool answerCall(const std::string& accountId,
+    bool acceptCall(const std::string& accountId,
                     const std::string& callId,
                     const std::vector<libjami::MediaMap>& mediaList = {});
-    bool answerCall(Call& call, const std::vector<libjami::MediaMap>& mediaList = {});
+    bool acceptCall(Call& call, const std::vector<libjami::MediaMap>& mediaList = {});
 
     /**
      * Handle incoming call and notify user
@@ -244,15 +243,16 @@ public:
     bool unHoldConference(const std::string& accountId, const std::string& confId);
 
     /**
-     * Add a participant to a conference
+     * Add a subcall to a conference
      * @param the call id
      * @param the conference id
      */
-    bool addParticipant(const std::string& accountId,
+    bool addSubCall(const std::string& accountId,
                         const std::string& callId,
                         const std::string& account2Id,
                         const std::string& confId);
-    bool addParticipant(Call& call, Conference& conference);
+    bool addSubCall(Call& call, Conference& conference);
+    void bindCallToConference(Call& call, Conference& conf);
 
     /**
      * Bind the main participant to a conference (mainly called on a double click action)
@@ -289,7 +289,7 @@ public:
      * Detach the local participant from curent conference.
      * Remote participants are placed in hold.
      */
-    bool detachLocalParticipant(const std::shared_ptr<Conference>& conf = {});
+    bool detachHost(const std::shared_ptr<Conference>& conf = {});
 
     /**
      * Remove the conference participant from a conference
@@ -383,8 +383,6 @@ public:
      */
     void sendRegister(const std::string& accountId, bool enable);
 
-    bool isPasswordValid(const std::string& accountID, const std::string& password);
-
     uint64_t sendTextMessage(const std::string& accountID,
                              const std::string& to,
                              const std::map<std::string, std::string>& payloads,
@@ -433,9 +431,16 @@ public:
      * @param details     The account parameters
      */
     void setAccountDetails(const std::string& accountID,
-                           const std::map<std::string, ::std::string>& details);
+                           const std::map<std::string, std::string>& details);
 
-    void setAccountActive(const std::string& accountID, bool active, bool shutdownConnections);
+    void setAccountActive(const std::string& accountID,
+                          bool active,
+                          bool shutdownConnections);
+    void loadAccountAndConversation(const std::string& accountId,
+                                    bool loadAll,
+                                    const std::string& convId);
+
+    std::mt19937_64 getSeededRandomEngine();
 
     /**
      * Return a new random accountid that is not present in the list
@@ -633,7 +638,7 @@ public:
     void playDtmf(char code);
 
     /**
-     * Handle played sound when a call can not be conpleted because of a busy recipient
+     * Handle played sound when a call is unable to be completed because of a busy recipient
      */
     void callBusy(Call& call);
 
@@ -764,6 +769,11 @@ public:
      */
     void registerAccounts();
 
+    /**
+     * Send registration for one account
+     */
+    void registerAccount(const std::string& accountID, const std::string& convId = {});
+
     /**
      * Send unregister for all enabled accounts
      */
@@ -783,12 +793,12 @@ public:
 
     CallFactory callFactory;
 
-    IceTransportFactory& getIceTransportFactory();
-
-    ScheduledExecutor& scheduler();
+    const std::shared_ptr<dhtnet::IceTransportFactory>& getIceTransportFactory();
 
     std::shared_ptr<asio::io_context> ioContext() const;
+    std::shared_ptr<dhtnet::upnp::UPnPContext> upnpContext() const;
 
+    ScheduledExecutor& scheduler();
     std::shared_ptr<Task> scheduleTask(std::function<void()>&& task,
                                        std::chrono::steady_clock::time_point when,
                                        const char* filename = CURRENT_FILENAME(),
@@ -837,9 +847,9 @@ public:
     std::shared_ptr<video::SinkClient> getSinkClient(const std::string& id);
 
 #endif // ENABLE_VIDEO
-    VideoManager& getVideoManager() const;
+    VideoManager* getVideoManager() const;
 
-    std::atomic<unsigned> dhtLogLevel {0}; // default = disable
+    unsigned dhtLogLevel {0}; // default = disable
     AccountFactory accountFactory;
 
     std::vector<libjami::Message> getLastMessages(const std::string& accountID,
@@ -856,9 +866,9 @@ public:
      * @param conversationId    Related conversation
      * @return std::optional<std::weak_ptr<ChannelSocket>> the related socket
      */
-    std::shared_ptr<ChannelSocket> gitSocket(const std::string_view accountId,
-                                             const std::string_view deviceId,
-                                             const std::string_view conversationId);
+    std::shared_ptr<dhtnet::ChannelSocket> gitSocket(std::string_view accountId,
+                                                     std::string_view deviceId,
+                                                     std::string_view conversationId);
 
     void setDefaultModerator(const std::string& accountID, const std::string& peerURI, bool state);
     std::vector<std::string> getDefaultModerators(const std::string& accountID);
@@ -870,7 +880,7 @@ public:
     void insertGitTransport(git_smart_subtransport* tr, std::unique_ptr<P2PSubTransport>&& sub);
     void eraseGitTransport(git_smart_subtransport* tr);
 
-    tls::CertificateStore& certStore(const std::string& accountId) const;
+    dhtnet::tls::CertificateStore& certStore(const std::string& accountId) const;
 
 private:
     Manager();
@@ -896,13 +906,9 @@ private:
 // Helper to install a callback to be called once by the main event loop
 template<typename Callback>
 static void
-runOnMainThread(Callback&& cb,
-                const char* filename = CURRENT_FILENAME(),
-                uint32_t linum = CURRENT_LINE())
+runOnMainThread(Callback&& cb)
 {
-    Manager::instance().scheduler().run([cb = std::forward<Callback>(cb)]() mutable { cb(); },
-                                        filename,
-                                        linum);
+    Manager::instance().scheduler().run([cb = std::forward<Callback>(cb)]() mutable { cb(); });
 }
 
 } // namespace jami
