@@ -1,21 +1,18 @@
 /*
- *  Copyright (C) 2004-2023 Savoir-faire Linux Inc.
+ *  Copyright (C) 2004-2025 Savoir-faire Linux Inc.
  *
- *  Author: Alexandre Savard <alexandre.savard@savoirfairelinux.com>
- *
- *  This program is free software; you can redistribute it and/or modify
+ *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 3 of the License, or
+ *  the Free Software Foundation, either version 3 of the License, or
  *  (at your option) any later version.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA.
+ *  along with this program. If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "opensllayer.h"
@@ -54,7 +51,7 @@ OpenSLLayer::startStream(AudioDeviceType stream)
 {
     using namespace std::placeholders;
 
-    std::lock_guard<std::mutex> lock(mutex_);
+    std::lock_guard lock(mutex_);
     if (!engineObject_)
         initAudioEngine();
 
@@ -93,7 +90,7 @@ OpenSLLayer::startStream(AudioDeviceType stream)
         }
     } else if (stream == AudioDeviceType::CAPTURE) {
         if (not recorder_) {
-            std::lock_guard<std::mutex> lck(recMtx);
+            std::lock_guard lck(recMtx);
             try {
                 recorder_.reset(
                     new opensl::AudioRecorder(hardwareFormat_, hardwareBuffSize_, engineInterface_));
@@ -115,7 +112,7 @@ OpenSLLayer::startStream(AudioDeviceType stream)
 void
 OpenSLLayer::stopStream(AudioDeviceType stream)
 {
-    std::lock_guard<std::mutex> lock(mutex_);
+    std::lock_guard lock(mutex_);
     if (!engineObject_)
         return; // bufs_ should be initialized
     JAMI_WARN("Stopping OpenSL audio layer for type %u", (unsigned) stream);
@@ -175,7 +172,7 @@ OpenSLLayer::initAudioEngine()
     std::vector<int32_t> hw_infos;
     hw_infos.reserve(4);
     emitSignal<libjami::ConfigurationSignal::GetHardwareAudioFormat>(&hw_infos);
-    hardwareFormat_ = AudioFormat(hw_infos[0], 1); // Mono on Android
+    hardwareFormat_ = AudioFormat(hw_infos[0], 1, AV_SAMPLE_FMT_FLT); // Mono on Android
     hardwareBuffSize_ = hw_infos[1];
     hardwareFormatAvailable(hardwareFormat_, hardwareBuffSize_);
 
@@ -184,9 +181,9 @@ OpenSLLayer::initAudioEngine()
     SLASSERT((*engineObject_)->GetInterface(engineObject_, SL_IID_ENGINE, &engineInterface_));
 
     size_t bufSize = hardwareBuffSize_ * hardwareFormat_.getBytesPerFrame();
-    JAMI_DBG("OpenSL init: using buffer of %zu bytes to support %s with %zu samples per channel",
+    JAMI_LOG("OpenSL init: using buffer of {:d} bytes to support {} with {:d} samples per channel",
              bufSize,
-             hardwareFormat_.toString().c_str(),
+             hardwareFormat_.toString(),
              hardwareBuffSize_);
     bufs_ = allocateSampleBufs(BUF_COUNT * 3, bufSize);
     for (int i = 0; i < BUF_COUNT; i++)
@@ -266,8 +263,7 @@ OpenSLLayer::engineServicePlay()
     sample_buf* buf;
     while (player_ and freePlayBufQueue_.front(&buf)) {
         if (auto dat = getToPlay(hardwareFormat_, hardwareBuffSize_)) {
-            buf->size_ = dat->pointer()->nb_samples * dat->pointer()->ch_layout.nb_channels
-                         * sizeof(AudioSample);
+            buf->size_ = dat->pointer()->nb_samples * dat->getFormat().getBytesPerFrame();
             if (buf->size_ > buf->cap_) {
                 JAMI_ERR("buf->size_(%zu) > buf->cap_(%zu)", buf->size_, buf->cap_);
                 break;
@@ -279,9 +275,7 @@ OpenSLLayer::engineServicePlay()
                          dat->pointer()->nb_samples);
                 break;
             }
-            std::copy_n((const AudioSample*) dat->pointer()->data[0],
-                        dat->pointer()->nb_samples,
-                        (AudioSample*) buf->buf_);
+            memcpy(buf->buf_, dat->pointer()->data[0], buf->size_);
             if (!playBufQueue_.push(buf)) {
                 JAMI_WARN("playThread player_ PLAY_KICKSTART_BUFFER_COUNT 1");
                 break;
@@ -299,8 +293,7 @@ OpenSLLayer::engineServiceRing()
     sample_buf* buf;
     while (ringtone_ and freeRingBufQueue_.front(&buf)) {
         if (auto dat = getToRing(hardwareFormat_, hardwareBuffSize_)) {
-            buf->size_ = dat->pointer()->nb_samples * dat->pointer()->ch_layout.nb_channels
-                         * sizeof(AudioSample);
+            buf->size_ = dat->pointer()->nb_samples * dat->getFormat().getBytesPerFrame();
             if (buf->size_ > buf->cap_) {
                 JAMI_ERR("buf->size_(%zu) > buf->cap_(%zu)", buf->size_, buf->cap_);
                 break;
@@ -312,9 +305,7 @@ OpenSLLayer::engineServiceRing()
                          dat->pointer()->nb_samples);
                 break;
             }
-            std::copy_n((const AudioSample*) dat->pointer()->data[0],
-                        dat->pointer()->nb_samples,
-                        (AudioSample*) buf->buf_);
+            memcpy(buf->buf_, dat->pointer()->data[0], buf->size_);
             if (!ringBufQueue_.push(buf)) {
                 JAMI_WARN("playThread ringtone_ PLAY_KICKSTART_BUFFER_COUNT 1");
                 break;
@@ -342,7 +333,7 @@ OpenSLLayer::startAudioCapture()
     if (recThread.joinable())
         return;
     recThread = std::thread([&]() {
-        std::unique_lock<std::mutex> lck(recMtx);
+        std::unique_lock lck(recMtx);
         if (recorder_)
             recorder_->start();
         recordChanged(true);
@@ -359,9 +350,7 @@ OpenSLLayer::startAudioCapture()
                     if (isCaptureMuted_)
                         libav_utils::fillWithSilence(out->pointer());
                     else
-                        std::copy_n((const AudioSample*) buf->buf_,
-                                    nb_samples,
-                                    (AudioSample*) out->pointer()->data[0]);
+                        memcpy(out->pointer()->data[0], buf->buf_, buf->size_);
                     putRecorded(std::move(out));
                 }
                 buf->size_ = 0;
@@ -380,7 +369,7 @@ OpenSLLayer::stopAudioCapture()
     JAMI_DBG("Stop audio capture");
 
     {
-        std::lock_guard<std::mutex> lck(recMtx);
+        std::lock_guard lck(recMtx);
         if (recorder_) {
             recorder_->stop();
             recorder_.reset();
