@@ -1,36 +1,32 @@
 /*
- *  Copyright (C) 2004-2023 Savoir-faire Linux Inc.
+ *  Copyright (C) 2004-2025 Savoir-faire Linux Inc.
  *
- *  Author: Edric Ladent-Milaret <edric.ladent-milaret@savoirfairelinux.com>
- *  Author: Guillaume Roguez <guillaume.roguez@savoirfairelinux.com>
- *  Author: Andreas Traczyk <andreas.traczyk@savoirfairelinux.com>
- *
- *  This program is free software; you can redistribute it and/or modify
+ *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 3 of the License, or
+ *  the Free Software Foundation, either version 3 of the License, or
  *  (at your option) any later version.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA.
+ *  along with this program. If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "portaudiolayer.h"
-#include "manager.h"
-#include "noncopyable.h"
+
 #include "audio/resampler.h"
-#include "audio/ringbufferpool.h"
-#include "audio/ringbuffer.h"
-#include "audio/audioloop.h"
+#include "audio/portaudio/audio_device_monitor.h"
+#include "manager.h"
+#include "preferences.h"
 
 #include <portaudio.h>
+
+#include <windows.h>
+
 #include <algorithm>
-#include <cmath>
 
 namespace jami {
 
@@ -66,30 +62,38 @@ struct PortAudioLayer::PortAudioLayerImpl
     bool outputInitialized_ {false};
 
     std::array<PaStream*, static_cast<int>(Direction::End)> streams_;
+    mutable std::mutex streamsMutex_;
+    bool paStopStream(Direction streamDirection);
+    bool hasFullDuplexStream() const;
+
+    AudioDeviceNotificationClientPtr audioDeviceNotificationClient_;
+    // The following flag used to debounce the device state changes,
+    // as default-device change events often follow plug/unplug events.
+    std::atomic<bool> restartRequestPending_ = false;
 
     int paOutputCallback(PortAudioLayer& parent,
-                         const AudioSample* inputBuffer,
-                         AudioSample* outputBuffer,
+                         const float* inputBuffer,
+                         float* outputBuffer,
                          unsigned long framesPerBuffer,
                          const PaStreamCallbackTimeInfo* timeInfo,
                          PaStreamCallbackFlags statusFlags);
 
     int paInputCallback(PortAudioLayer& parent,
-                        const AudioSample* inputBuffer,
-                        AudioSample* outputBuffer,
+                        const float* inputBuffer,
+                        float* outputBuffer,
                         unsigned long framesPerBuffer,
                         const PaStreamCallbackTimeInfo* timeInfo,
                         PaStreamCallbackFlags statusFlags);
 
     int paIOCallback(PortAudioLayer& parent,
-                     const AudioSample* inputBuffer,
-                     AudioSample* outputBuffer,
+                     const float* inputBuffer,
+                     float* outputBuffer,
                      unsigned long framesPerBuffer,
                      const PaStreamCallbackTimeInfo* timeInfo,
                      PaStreamCallbackFlags statusFlags);
 };
 
-//##################################################################################################
+// ##################################################################################################
 
 PortAudioLayer::PortAudioLayer(const AudioPreference& pref)
     : AudioLayer {pref}
@@ -108,6 +112,9 @@ PortAudioLayer::PortAudioLayer(const AudioPreference& pref)
         deviceInfo = Pa_GetDeviceInfo(i);
         JAMI_DBG("PortAudio device: %d, %s", i, deviceInfo->name);
     }
+
+    // Notify of device changes in case this layer was reset based on a hotplug event.
+    devicesChanged();
 }
 
 PortAudioLayer::~PortAudioLayer()
@@ -140,9 +147,7 @@ PortAudioLayer::getAudioDeviceIndex(const std::string& name, AudioDeviceType typ
 std::string
 PortAudioLayer::getAudioDeviceName(int index, AudioDeviceType type) const
 {
-    (void) index;
-    (void) type;
-    return {};
+    return pimpl_->getDeviceNameByType(index, type);
 }
 
 int
@@ -173,7 +178,7 @@ PortAudioLayer::startStream(AudioDeviceType stream)
     }
 
     auto startPlayback = [this](bool fullDuplexMode = false) -> bool {
-        std::unique_lock<std::mutex> lock(mutex_);
+        std::unique_lock lock(mutex_);
         if (status_.load() != Status::Idle)
             return false;
         bool ret {false};
@@ -183,6 +188,7 @@ PortAudioLayer::startStream(AudioDeviceType stream)
             ret = pimpl_->initOutputStream(*this);
         if (ret) {
             status_.store(Status::Started);
+            startedCv_.notify_all();
             lock.unlock();
             flushUrgent();
             flushMain();
@@ -210,71 +216,64 @@ PortAudioLayer::startStream(AudioDeviceType stream)
 void
 PortAudioLayer::stopStream(AudioDeviceType stream)
 {
-    auto stopPaStream = [](PaStream* stream) -> bool {
-        if (!stream || Pa_IsStreamStopped(stream) != paNoError)
-            return false;
-        auto err = Pa_StopStream(stream);
-        if (err != paNoError) {
-            JAMI_ERR("Pa_StopStream error : %s", Pa_GetErrorText(err));
-            return false;
-        }
-        err = Pa_CloseStream(stream);
-        if (err != paNoError) {
-            JAMI_ERR("Pa_CloseStream error : %s", Pa_GetErrorText(err));
-            return false;
-        }
-        return true;
-    };
-
-    auto stopPlayback = [this, &stopPaStream](bool fullDuplexMode = false) -> bool {
-        std::lock_guard<std::mutex> lock(mutex_);
-        if (status_.load() != Status::Started)
-            return false;
-        bool stopped = false;
-        if (fullDuplexMode)
-            stopped = stopPaStream(pimpl_->streams_[Direction::IO]);
-        else
-            stopped = stopPaStream(pimpl_->streams_[Direction::Output]);
-        if (stopped)
-            status_.store(Status::Idle);
-        return stopped;
-    };
+    std::lock_guard lock(mutex_);
+    if (status_.load() != Status::Started)
+        return;
 
     bool stopped = false;
+    bool outputStopped = false;
     switch (stream) {
     case AudioDeviceType::ALL:
-        if (pimpl_->streams_[Direction::IO]) {
-            stopped = stopPlayback(true);
+        if (pimpl_->hasFullDuplexStream()) {
+            stopped = pimpl_->paStopStream(Direction::IO);
+            JAMI_DBG("PortAudioLayer full-duplex stream stopped");
+            if (stopped) {
+                playbackChanged(false);
+                recordChanged(false);
+                outputStopped = true;
+            }
         } else {
-            stopped = stopPaStream(pimpl_->streams_[Direction::Input]) && stopPlayback();
+            if (pimpl_->paStopStream(Direction::Output)) {
+                playbackChanged(false);
+                stopped = true;
+                outputStopped = true;
+                JAMI_DBG("PortAudioLayer output stream stopped");
+            }
+            if (pimpl_->paStopStream(Direction::Input)) {
+                recordChanged(false);
+                stopped = true;
+                JAMI_DBG("PortAudioLayer input stream stopped");
+            }
         }
-        if (stopped) {
-            recordChanged(false);
-            playbackChanged(false);
-            JAMI_DBG("PortAudioLayer I/O streams stopped");
-        } else
-            return;
         break;
     case AudioDeviceType::CAPTURE:
-        if (stopPaStream(pimpl_->streams_[Direction::Input])) {
+        if (pimpl_->paStopStream(Direction::Input)) {
             recordChanged(false);
             JAMI_DBG("PortAudioLayer input stream stopped");
-        } else
-            return;
+            stopped = true;
+        }
         break;
     case AudioDeviceType::PLAYBACK:
     case AudioDeviceType::RINGTONE:
-        if (stopPlayback()) {
+        if (pimpl_->paStopStream(Direction::Output)) {
             playbackChanged(false);
+            stopped = true;
+            outputStopped = true;
             JAMI_DBG("PortAudioLayer output stream stopped");
-        } else
-            return;
+        }
         break;
     }
 
-    // Flush the ring buffers
-    flushUrgent();
-    flushMain();
+    // Flush the ring buffers if any streams were stopped
+    if (stopped) {
+        JAMI_DBG("PortAudioLayer streams stopped, flushing buffers");
+        flushUrgent();
+        flushMain();
+        if (outputStopped) {
+            status_.store(Status::Idle);
+            startedCv_.notify_all();
+        }
+    }
 }
 
 void
@@ -296,14 +295,34 @@ PortAudioLayer::updatePreference(AudioPreference& preference, int index, AudioDe
     }
 }
 
-//##################################################################################################
+// ##################################################################################################
 
 PortAudioLayer::PortAudioLayerImpl::PortAudioLayerImpl(PortAudioLayer& parent,
                                                        const AudioPreference& pref)
     : deviceRecord_ {pref.getPortAudioDeviceRecord()}
     , devicePlayback_ {pref.getPortAudioDevicePlayback()}
     , deviceRingtone_ {pref.getPortAudioDeviceRingtone()}
+    , audioDeviceNotificationClient_ {new AudioDeviceNotificationClient}
 {
+    // Set up our callback to restart the layer on any device event
+    audioDeviceNotificationClient_->setDeviceEventCallback(
+        [this, &parent](const std::string& deviceName, const DeviceEventType event) {
+            JAMI_LOG("PortAudioLayer device event: {}, {}",
+                     deviceName.c_str(),
+                     to_string(event).c_str());
+            // Here we want to debounce the device events as a DefaultChanged could
+            // follow a DeviceAdded event and we don't want to restart the layer twice
+            if (!restartRequestPending_.exchange(true)) {
+                std::thread([] {
+                    // First wait for the debounce period to pass, allowing for multiple events
+                    // to be grouped together (e.g. DeviceAdded -> DefaultChanged).
+                    std::this_thread::sleep_for(std::chrono::milliseconds(300));
+                    auto currentAudioManager = Manager::instance().getAudioManager();
+                    Manager::instance().setAudioPlugin(currentAudioManager);
+                }).detach();
+            }
+        });
+
     init(parent);
 }
 
@@ -325,7 +344,7 @@ PortAudioLayer::PortAudioLayerImpl::initInput(PortAudioLayer& parent)
     const auto inputDeviceInfo = Pa_GetDeviceInfo(apiIndex);
     if (!inputDeviceInfo) {
         // this represents complete failure after attempting a fallback to default
-        JAMI_WARN("PortAudioLayer could not initialize input");
+        JAMI_WARN("PortAudioLayer was unable to initialize input");
         deviceRecord_.clear();
         inputInitialized_ = true;
         return;
@@ -335,7 +354,7 @@ PortAudioLayer::PortAudioLayerImpl::initInput(PortAudioLayer& parent)
     // internal index to paNoDevice and reenter in an attempt to set the default
     // communications device
     if (inputDeviceInfo->maxInputChannels <= 0) {
-        JAMI_WARN("PortAudioLayer could not initialize input, falling back to default device");
+        JAMI_WARN("PortAudioLayer was unable to initialize input, falling back to default device");
         deviceRecord_.clear();
         return initInput(parent);
     }
@@ -343,9 +362,11 @@ PortAudioLayer::PortAudioLayerImpl::initInput(PortAudioLayer& parent)
     // at this point, the device is of the correct type and can be opened
     parent.audioInputFormat_.sample_rate = inputDeviceInfo->defaultSampleRate;
     parent.audioInputFormat_.nb_channels = inputDeviceInfo->maxInputChannels;
+    parent.audioInputFormat_.sampleFormat = AV_SAMPLE_FMT_FLTP;
     parent.hardwareInputFormatAvailable(parent.audioInputFormat_);
-    JAMI_DBG("PortAudioLayer initialized input: %s {%d Hz, %d channels}",
+    JAMI_DBG("PortAudio input device: %s (native: %.0f Hz, using: %d Hz, %d channels)",
              inputDeviceInfo->name,
+             inputDeviceInfo->defaultSampleRate,
              parent.audioInputFormat_.sample_rate,
              parent.audioInputFormat_.nb_channels);
     inputInitialized_ = true;
@@ -364,7 +385,7 @@ PortAudioLayer::PortAudioLayerImpl::initOutput(PortAudioLayer& parent)
     const auto outputDeviceInfo = Pa_GetDeviceInfo(apiIndex);
     if (!outputDeviceInfo) {
         // this represents complete failure after attempting a fallback to default
-        JAMI_WARN("PortAudioLayer could not initialize output");
+        JAMI_WARN("PortAudioLayer was unable to initialize output");
         devicePlayback_.clear();
         outputInitialized_ = true;
         return;
@@ -374,7 +395,7 @@ PortAudioLayer::PortAudioLayerImpl::initOutput(PortAudioLayer& parent)
     // internal index to paNoDevice and reenter in an attempt to set the default
     // communications device
     if (outputDeviceInfo->maxOutputChannels <= 0) {
-        JAMI_WARN("PortAudioLayer could not initialize output, falling back to default device");
+        JAMI_WARN("PortAudioLayer was unable to initialize output, falling back to default device");
         devicePlayback_.clear();
         return initOutput(parent);
     }
@@ -382,9 +403,11 @@ PortAudioLayer::PortAudioLayerImpl::initOutput(PortAudioLayer& parent)
     // at this point, the device is of the correct type and can be opened
     parent.audioFormat_.sample_rate = outputDeviceInfo->defaultSampleRate;
     parent.audioFormat_.nb_channels = outputDeviceInfo->maxOutputChannels;
+    parent.audioFormat_.sampleFormat = AV_SAMPLE_FMT_FLTP;
     parent.hardwareFormatAvailable(parent.audioFormat_);
-    JAMI_DBG("PortAudioLayer initialized output: %s {%d Hz, %d channels}",
+    JAMI_DBG("PortAudio output device: %s (native: %.0f Hz, using: %d Hz, %d channels)",
              outputDeviceInfo->name,
+             outputDeviceInfo->defaultSampleRate,
              parent.audioFormat_.sample_rate,
              parent.audioFormat_.nb_channels);
     outputInitialized_ = true;
@@ -398,7 +421,7 @@ PortAudioLayer::PortAudioLayerImpl::init(PortAudioLayer& parent)
     auto apiIndex = Pa_GetDefaultHostApi();
     auto apiInfo = Pa_GetHostApiInfo(apiIndex);
     if (err != paNoError || apiInfo == nullptr) {
-        JAMI_ERR("PortAudioLayer error : %s", Pa_GetErrorText(err));
+        JAMI_ERR("PortAudioLayer error: %s", Pa_GetErrorText(err));
         terminate();
         return;
     }
@@ -409,6 +432,7 @@ PortAudioLayer::PortAudioLayerImpl::init(PortAudioLayer& parent)
     initInput(parent);
     initOutput(parent);
 
+    std::lock_guard lock(streamsMutex_);
     std::fill(std::begin(streams_), std::end(streams_), nullptr);
 }
 
@@ -418,7 +442,7 @@ PortAudioLayer::PortAudioLayerImpl::getDevicesByType(AudioDeviceType type) const
     std::vector<std::string> devices;
     auto numDevices = Pa_GetDeviceCount();
     if (numDevices < 0)
-        JAMI_ERR("PortAudioLayer error : %s", Pa_GetErrorText(numDevices));
+        JAMI_ERR("PortAudioLayer error: %s", Pa_GetErrorText(numDevices));
     else {
         for (int i = 0; i < numDevices; i++) {
             const auto deviceInfo = Pa_GetDeviceInfo(i);
@@ -472,24 +496,26 @@ PaDeviceIndex
 PortAudioLayer::PortAudioLayerImpl::getApiIndexByType(AudioDeviceType type)
 {
     auto numDevices = Pa_GetDeviceCount();
-    if (numDevices < 0)
-        JAMI_ERR("PortAudioLayer error : %s", Pa_GetErrorText(numDevices));
-    else {
+    if (numDevices < 0) {
+        JAMI_ERR("PortAudioLayer error: %s", Pa_GetErrorText(numDevices));
+        return paNoDevice;
+    } else {
         std::string_view toMatch = (type == AudioDeviceType::CAPTURE
                                         ? deviceRecord_
                                         : (type == AudioDeviceType::PLAYBACK ? devicePlayback_
                                                                              : deviceRingtone_));
-        if (toMatch.empty())
-            return type == AudioDeviceType::CAPTURE ? Pa_GetDefaultCommInputDevice()
-                                                    : Pa_GetDefaultCommOutputDevice();
-        for (int i = 0; i < numDevices; ++i) {
-            if (const auto deviceInfo = Pa_GetDeviceInfo(i)) {
-                if (deviceInfo->name == toMatch)
-                    return i;
+        if (!toMatch.empty()) {
+            for (int i = 0; i < numDevices; ++i) {
+                if (const auto deviceInfo = Pa_GetDeviceInfo(i)) {
+                    if (deviceInfo->name == toMatch)
+                        return i;
+                }
             }
         }
     }
-    return paNoDevice;
+    // If nothing was found, return the default device
+    return type == AudioDeviceType::CAPTURE ? Pa_GetDefaultCommInputDevice()
+                                            : Pa_GetDefaultCommOutputDevice();
 }
 
 std::string
@@ -515,89 +541,93 @@ PortAudioLayer::PortAudioLayerImpl::terminate() const
     JAMI_DBG("PortAudioLayer terminate.");
     auto err = Pa_Terminate();
     if (err != paNoError)
-        JAMI_ERR("PortAudioLayer error : %s", Pa_GetErrorText(err));
+        JAMI_ERR("PortAudioLayer error: %s", Pa_GetErrorText(err));
 }
 
-static void
-openStreamDevice(PaStream** stream,
-                 PaDeviceIndex device,
-                 Direction direction,
-                 PaStreamCallback* callback,
-                 void* user_data)
+// Unified PortAudio stream opener for input, output, or full-duplex
+static PaError
+openPaStream(PaStream** stream,
+             PaDeviceIndex inputDeviceIndex,
+             PaDeviceIndex outputDeviceIndex,
+             PaStreamCallback* callback,
+             void* user_data)
 {
-    auto is_out = direction == Direction::Output;
-    auto device_info = Pa_GetDeviceInfo(device);
+    const bool hasInput = inputDeviceIndex != paNoDevice;
+    const bool hasOutput = outputDeviceIndex != paNoDevice;
 
-    PaStreamParameters params;
-    params.device = device;
-    params.channelCount = is_out ? device_info->maxOutputChannels : device_info->maxInputChannels;
-    params.sampleFormat = paInt16;
-    params.suggestedLatency = is_out ? device_info->defaultLowOutputLatency
-                                     : device_info->defaultLowInputLatency;
-    params.hostApiSpecificStreamInfo = nullptr;
-
-    auto err = Pa_OpenStream(stream,
-                             is_out ? nullptr : &params,
-                             is_out ? &params : nullptr,
-                             device_info->defaultSampleRate,
-                             paFramesPerBufferUnspecified,
-                             paNoFlag,
-                             callback,
-                             user_data);
-
-    if (err != paNoError)
-        JAMI_ERR("PortAudioLayer error : %s", Pa_GetErrorText(err));
-}
-
-static void
-openFullDuplexStream(PaStream** stream,
-                     PaDeviceIndex inputDeviceIndex,
-                     PaDeviceIndex ouputDeviceIndex,
-                     PaStreamCallback* callback,
-                     void* user_data)
-{
-    auto input_device_info = Pa_GetDeviceInfo(inputDeviceIndex);
-    auto output_device_info = Pa_GetDeviceInfo(ouputDeviceIndex);
+    const PaDeviceInfo* inputInfo = hasInput ? Pa_GetDeviceInfo(inputDeviceIndex) : nullptr;
+    const PaDeviceInfo* outputInfo = hasOutput ? Pa_GetDeviceInfo(outputDeviceIndex) : nullptr;
 
     PaStreamParameters inputParams;
-    inputParams.device = inputDeviceIndex;
-    inputParams.channelCount = input_device_info->maxInputChannels;
-    inputParams.sampleFormat = paInt16;
-    inputParams.suggestedLatency = input_device_info->defaultLowInputLatency;
-    inputParams.hostApiSpecificStreamInfo = nullptr;
-
     PaStreamParameters outputParams;
-    outputParams.device = ouputDeviceIndex;
-    outputParams.channelCount = output_device_info->maxOutputChannels;
-    outputParams.sampleFormat = paInt16;
-    outputParams.suggestedLatency = output_device_info->defaultLowOutputLatency;
-    outputParams.hostApiSpecificStreamInfo = nullptr;
+    PaStreamParameters* inputParamsPtr = nullptr;
+    PaStreamParameters* outputParamsPtr = nullptr;
 
+    if (hasInput && inputInfo) {
+        inputParams.device = inputDeviceIndex;
+        inputParams.channelCount = inputInfo->maxInputChannels;
+        inputParams.sampleFormat = paFloat32 | paNonInterleaved;
+        inputParams.suggestedLatency = inputInfo->defaultLowInputLatency;
+        inputParams.hostApiSpecificStreamInfo = nullptr;
+        inputParamsPtr = &inputParams;
+    }
+    if (hasOutput && outputInfo) {
+        outputParams.device = outputDeviceIndex;
+        outputParams.channelCount = outputInfo->maxOutputChannels;
+        outputParams.sampleFormat = paFloat32 | paNonInterleaved;
+        outputParams.suggestedLatency = outputInfo->defaultLowOutputLatency;
+        outputParams.hostApiSpecificStreamInfo = nullptr;
+        outputParamsPtr = &outputParams;
+    }
+
+    // Choose a working sample rate
+    double sampleRate = 0.0;
+    if (inputParamsPtr && outputParamsPtr) {
+        sampleRate = std::min(inputInfo->defaultSampleRate, outputInfo->defaultSampleRate);
+    } else if (inputParamsPtr) {
+        sampleRate = inputInfo->defaultSampleRate;
+    } else if (outputParamsPtr) {
+        sampleRate = outputInfo->defaultSampleRate;
+    }
+
+    unsigned long framesPerBuffer = (unsigned long) std::max(1.0, sampleRate / 100.0); // ~10 ms
     auto err = Pa_OpenStream(stream,
-                             &inputParams,
-                             &outputParams,
-                             std::min(input_device_info->defaultSampleRate,
-                                      input_device_info->defaultSampleRate),
-                             paFramesPerBufferUnspecified,
+                             inputParamsPtr,
+                             outputParamsPtr,
+                             sampleRate,
+                             framesPerBuffer,
                              paNoFlag,
                              callback,
                              user_data);
+    if (err != paNoError) {
+        JAMI_ERR("PortAudioLayer error: %s", Pa_GetErrorText(err));
+    }
+    return err;
+}
 
-    if (err != paNoError)
-        JAMI_ERR("PortAudioLayer error : %s", Pa_GetErrorText(err));
+static bool
+startPaStream(PaStream* stream)
+{
+    auto err = Pa_StartStream(stream);
+    if (err != paNoError) {
+        JAMI_ERR("PortAudioLayer error: %s", Pa_GetErrorText(err));
+        return false;
+    }
+    return true;
 }
 
 bool
 PortAudioLayer::PortAudioLayerImpl::initInputStream(PortAudioLayer& parent)
 {
     JAMI_DBG("Open PortAudio Input Stream");
+    std::lock_guard lock(streamsMutex_);
     auto& stream = streams_[Direction::Input];
     auto apiIndex = getApiIndexByType(AudioDeviceType::CAPTURE);
     if (apiIndex != paNoDevice) {
-        openStreamDevice(
-            &streams_[Direction::Input],
+        auto err = openPaStream(
+            &stream,
             apiIndex,
-            Direction::Input,
+            paNoDevice,
             [](const void* inputBuffer,
                void* outputBuffer,
                unsigned long framesPerBuffer,
@@ -606,24 +636,24 @@ PortAudioLayer::PortAudioLayerImpl::initInputStream(PortAudioLayer& parent)
                void* userData) -> int {
                 auto layer = static_cast<PortAudioLayer*>(userData);
                 return layer->pimpl_->paInputCallback(*layer,
-                                                      static_cast<const AudioSample*>(inputBuffer),
-                                                      static_cast<AudioSample*>(outputBuffer),
+                                                      static_cast<const float*>(inputBuffer),
+                                                      static_cast<float*>(outputBuffer),
                                                       framesPerBuffer,
                                                       timeInfo,
                                                       statusFlags);
             },
             &parent);
+        if (err != paNoError) {
+            return false;
+        }
     } else {
         JAMI_ERR("Error: No valid input device. There will be no mic.");
         return false;
     }
 
     JAMI_DBG("Starting PortAudio Input Stream");
-    auto err = Pa_StartStream(stream);
-    if (err != paNoError) {
-        JAMI_ERR("PortAudioLayer error : %s", Pa_GetErrorText(err));
+    if (!startPaStream(stream))
         return false;
-    }
 
     parent.recordChanged(true);
     return true;
@@ -633,13 +663,14 @@ bool
 PortAudioLayer::PortAudioLayerImpl::initOutputStream(PortAudioLayer& parent)
 {
     JAMI_DBG("Open PortAudio Output Stream");
+    std::lock_guard lock(streamsMutex_);
     auto& stream = streams_[Direction::Output];
     auto apiIndex = getApiIndexByType(AudioDeviceType::PLAYBACK);
     if (apiIndex != paNoDevice) {
-        openStreamDevice(
+        auto err = openPaStream(
             &stream,
+            paNoDevice,
             apiIndex,
-            Direction::Output,
             [](const void* inputBuffer,
                void* outputBuffer,
                unsigned long framesPerBuffer,
@@ -648,24 +679,24 @@ PortAudioLayer::PortAudioLayerImpl::initOutputStream(PortAudioLayer& parent)
                void* userData) -> int {
                 auto layer = static_cast<PortAudioLayer*>(userData);
                 return layer->pimpl_->paOutputCallback(*layer,
-                                                       static_cast<const AudioSample*>(inputBuffer),
-                                                       static_cast<AudioSample*>(outputBuffer),
+                                                       static_cast<const float*>(inputBuffer),
+                                                       static_cast<float*>(outputBuffer),
                                                        framesPerBuffer,
                                                        timeInfo,
                                                        statusFlags);
             },
             &parent);
+        if (err != paNoError) {
+            return false;
+        }
     } else {
         JAMI_ERR("Error: No valid output device. There will be no sound.");
         return false;
     }
 
     JAMI_DBG("Starting PortAudio Output Stream");
-    auto err = Pa_StartStream(stream);
-    if (err != paNoError) {
-        JAMI_ERR("PortAudioLayer error : %s", Pa_GetErrorText(err));
+    if (!startPaStream(stream))
         return false;
-    }
 
     parent.playbackChanged(true);
     return true;
@@ -682,8 +713,9 @@ PortAudioLayer::PortAudioLayerImpl::initFullDuplexStream(PortAudioLayer& parent)
     }
 
     JAMI_DBG("Open PortAudio Full-duplex input/output stream");
+    std::lock_guard lock(streamsMutex_);
     auto& stream = streams_[Direction::IO];
-    openFullDuplexStream(
+    auto err = openPaStream(
         &stream,
         apiIndexRecord,
         apiIndexPlayback,
@@ -695,30 +727,67 @@ PortAudioLayer::PortAudioLayerImpl::initFullDuplexStream(PortAudioLayer& parent)
            void* userData) -> int {
             auto layer = static_cast<PortAudioLayer*>(userData);
             return layer->pimpl_->paIOCallback(*layer,
-                                               static_cast<const AudioSample*>(inputBuffer),
-                                               static_cast<AudioSample*>(outputBuffer),
+                                               static_cast<const float*>(inputBuffer),
+                                               static_cast<float*>(outputBuffer),
                                                framesPerBuffer,
                                                timeInfo,
                                                statusFlags);
         },
         &parent);
-
-    JAMI_DBG("Start PortAudio I/O Streams");
-    auto err = Pa_StartStream(stream);
     if (err != paNoError) {
-        JAMI_ERR("PortAudioLayer error : %s", Pa_GetErrorText(err));
         return false;
     }
 
+    JAMI_DBG("Start PortAudio I/O Streams");
+    if (!startPaStream(stream))
+        return false;
+
     parent.recordChanged(true);
     parent.playbackChanged(true);
     return true;
 }
 
+bool
+PortAudioLayer::PortAudioLayerImpl::paStopStream(Direction streamDirection)
+{
+    std::lock_guard lock(streamsMutex_);
+    PaStream* paStream = streams_[streamDirection];
+    if (!paStream)
+        return false;
+    auto ret = Pa_IsStreamStopped(paStream);
+    if (ret == 1) {
+        JAMI_DBG("PortAudioLayer stream %d already stopped", streamDirection);
+        return true;
+    } else if (ret < 0) {
+        JAMI_ERR("Pa_IsStreamStopped error: %s", Pa_GetErrorText(ret));
+        return false;
+    }
+    auto err = Pa_StopStream(paStream);
+    if (err != paNoError) {
+        JAMI_ERR("Pa_StopStream error: %s", Pa_GetErrorText(err));
+        return false;
+    }
+    err = Pa_CloseStream(paStream);
+    if (err != paNoError) {
+        JAMI_ERR("Pa_CloseStream error: %s", Pa_GetErrorText(err));
+        return false;
+    }
+    JAMI_DBG("PortAudioLayer stream %d stopped", streamDirection);
+    streams_[streamDirection] = nullptr;
+    return true;
+};
+
+bool
+PortAudioLayer::PortAudioLayerImpl::hasFullDuplexStream() const
+{
+    std::lock_guard lock(streamsMutex_);
+    return streams_[Direction::IO] != nullptr;
+}
+
 int
 PortAudioLayer::PortAudioLayerImpl::paOutputCallback(PortAudioLayer& parent,
-                                                     const AudioSample* inputBuffer,
-                                                     AudioSample* outputBuffer,
+                                                     const float* inputBuffer,
+                                                     float* outputBuffer,
                                                      unsigned long framesPerBuffer,
                                                      const PaStreamCallbackTimeInfo* timeInfo,
                                                      PaStreamCallbackFlags statusFlags)
@@ -730,19 +799,28 @@ PortAudioLayer::PortAudioLayerImpl::paOutputCallback(PortAudioLayer& parent,
 
     auto toPlay = parent.getPlayback(parent.audioFormat_, framesPerBuffer);
     if (!toPlay) {
-        std::fill_n(outputBuffer, framesPerBuffer * parent.audioFormat_.nb_channels, 0);
+        // Fill silence for all channels in planar format
+        float** outputChannels = (float**) outputBuffer;
+        for (unsigned i = 0; i < parent.audioFormat_.nb_channels; ++i) {
+            std::fill_n(outputChannels[i], framesPerBuffer, 0.0f);
+        }
         return paContinue;
     }
 
-    auto nFrames = toPlay->pointer()->nb_samples * toPlay->pointer()->ch_layout.nb_channels;
-    std::copy_n((AudioSample*) toPlay->pointer()->extended_data[0], nFrames, outputBuffer);
+    auto numSamples = toPlay->pointer()->nb_samples;
+    auto channels = std::min<size_t>(parent.audioFormat_.nb_channels,
+                                     toPlay->pointer()->ch_layout.nb_channels);
+    float** outputChannels = (float**) outputBuffer;
+    for (size_t i = 0; i < channels; ++i) {
+        std::copy_n((float*) toPlay->pointer()->extended_data[i], numSamples, outputChannels[i]);
+    }
     return paContinue;
 }
 
 int
 PortAudioLayer::PortAudioLayerImpl::paInputCallback(PortAudioLayer& parent,
-                                                    const AudioSample* inputBuffer,
-                                                    AudioSample* outputBuffer,
+                                                    const float* inputBuffer,
+                                                    float* outputBuffer,
                                                     unsigned long framesPerBuffer,
                                                     const PaStreamCallbackTimeInfo* timeInfo,
                                                     PaStreamCallbackFlags statusFlags)
@@ -758,19 +836,25 @@ PortAudioLayer::PortAudioLayerImpl::paInputCallback(PortAudioLayer& parent,
     }
 
     auto inBuff = std::make_shared<AudioFrame>(parent.audioInputFormat_, framesPerBuffer);
-    auto nFrames = framesPerBuffer * parent.audioInputFormat_.nb_channels;
-    if (parent.isCaptureMuted_)
+    if (parent.isCaptureMuted_ || inputBuffer == nullptr) {
         libav_utils::fillWithSilence(inBuff->pointer());
-    else
-        std::copy_n(inputBuffer, nFrames, (AudioSample*) inBuff->pointer()->extended_data[0]);
+    } else {
+        auto channels = parent.audioInputFormat_.nb_channels;
+        float** inputChannels = (float**) inputBuffer;
+        for (size_t i = 0; i < channels; ++i) {
+            std::copy_n(inputChannels[i],
+                        framesPerBuffer,
+                        (float*) inBuff->pointer()->extended_data[i]);
+        }
+    }
     parent.putRecorded(std::move(inBuff));
     return paContinue;
 }
 
 int
 PortAudioLayer::PortAudioLayerImpl::paIOCallback(PortAudioLayer& parent,
-                                                 const AudioSample* inputBuffer,
-                                                 AudioSample* outputBuffer,
+                                                 const float* inputBuffer,
+                                                 float* outputBuffer,
                                                  unsigned long framesPerBuffer,
                                                  const PaStreamCallbackTimeInfo* timeInfo,
                                                  PaStreamCallbackFlags statusFlags)
