@@ -1,15 +1,14 @@
 /*
- *  Copyright (C) 2022-2024 Savoir-faire Linux Inc.
- *  Author: Sébastien Blin <sebastien.blin@savoirfairelinux.com>
+ *  Copyright (C) 2004-2025 Savoir-faire Linux Inc.
  *
- *  This program is free software; you can redistribute it and/or modify
+ *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 3 of the License, or
+ *  the Free Software Foundation, either version 3 of the License, or
  *  (at your option) any later version.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
@@ -40,6 +39,8 @@ namespace test {
 struct ConvData
 {
     std::string id {};
+    std::string callId {};
+    std::string confId {};
     bool requestReceived {false};
     bool needsHost {false};
     bool conferenceChanged {false};
@@ -87,6 +88,9 @@ private:
     void testNeedsHost();
     void testAudioOnly();
     void testJoinAfterMuteHost();
+    void testBusy();
+    void testDecline();
+    void testNoDevice();
 
     CPPUNIT_TEST_SUITE(ConversationCallTest);
     CPPUNIT_TEST(testActiveCalls);
@@ -101,6 +105,9 @@ private:
     CPPUNIT_TEST(testNeedsHost);
     CPPUNIT_TEST(testAudioOnly);
     CPPUNIT_TEST(testJoinAfterMuteHost);
+    CPPUNIT_TEST(testBusy);
+    CPPUNIT_TEST(testDecline);
+    CPPUNIT_TEST(testNoDevice);
     CPPUNIT_TEST_SUITE_END();
 };
 
@@ -204,6 +211,14 @@ ConversationCallTest::connectSignals()
                 aliceData_.conferenceChanged = true;
             cv.notify_one();
         }));
+    confHandlers.insert(libjami::exportable_callback<libjami::CallSignal::ConferenceCreated>(
+        [&](const std::string& accountId, const std::string&, const std::string& confId) {
+            if (accountId == aliceId)
+                aliceData_.confId = confId;
+            else if (accountId == bobId)
+                bobData_.confId = confId;
+            cv.notify_one();
+        }));
     confHandlers.insert(libjami::exportable_callback<libjami::CallSignal::ConferenceRemoved>(
         [&](const std::string& accountId, const std::string&) {
             if (accountId == aliceId)
@@ -229,15 +244,13 @@ ConversationCallTest::connectSignals()
                         carlaData_.hostState = state;
                 }
             } else if (accountId == bobId) {
+                bobData_.callId = callId;
                 bobData_.state = state;
             } else if (accountId == carlaId) {
                 carlaData_.state = state;
             }
             cv.notify_one();
         }));
-
-
-
     confHandlers.insert(libjami::exportable_callback<libjami::CallSignal::OnConferenceInfosUpdated>(
         [=](const std::string&,
             const std::vector<std::map<std::string, std::string>> participantsInfos) {
@@ -292,7 +305,7 @@ ConversationCallTest::testActiveCalls()
 
     // start call
     aliceData_.messages.clear();
-    auto callId = libjami::placeCallWithMedia(aliceId, "swarm:" + aliceData_.id, {});
+    libjami::placeCallWithMedia(aliceId, "swarm:" + aliceData_.id, {});
     // should get message
     cv.wait_for(lk, 30s, [&]() { return !aliceData_.conferenceChanged && !aliceData_.messages.empty(); });
     CPPUNIT_ASSERT(aliceData_.messages.rbegin()->type == "application/call-history+json");
@@ -302,10 +315,10 @@ ConversationCallTest::testActiveCalls()
 
     // hangup
     aliceData_.messages.clear();
-    Manager::instance().hangupCall(aliceId, callId);
+    Manager::instance().hangupConference(aliceId, aliceData_.confId);
 
     // should get message
-    cv.wait_for(lk, 30s, [&]() { return !aliceData_.messages.empty(); });
+    CPPUNIT_ASSERT(cv.wait_for(lk, 30s, [&]() { return !aliceData_.messages.empty(); }));
     CPPUNIT_ASSERT(aliceData_.messages.rbegin()->body.find("duration") != aliceData_.messages.rbegin()->body.end());
 
     // get active calls = 0
@@ -349,7 +362,7 @@ ConversationCallTest::testActiveCalls3Peers()
     aliceData_.messages.clear();
     bobData_.messages.clear();
     carlaData_.messages.clear();
-    auto callId = libjami::placeCallWithMedia(aliceId, "swarm:" + aliceData_.id, {});
+    libjami::placeCallWithMedia(aliceId, "swarm:" + aliceData_.id, {});
     auto lastCommitIsCall = [&](const auto& data) {
         return !data.messages.empty()
                && data.messages.rbegin()->body.at("type") == "application/call-history+json";
@@ -372,10 +385,10 @@ ConversationCallTest::testActiveCalls3Peers()
         return aliceData_.conferenceChanged && bobData_.hostState == "CURRENT";
     }));
     aliceData_.conferenceChanged = false;
-    // get 3 participants
+    // get 2 other participants
     libjami::placeCallWithMedia(carlaId, destination, {});
     CPPUNIT_ASSERT(cv.wait_for(lk, 30s, [&]() {
-        return aliceData_.conferenceChanged && carlaData_.hostState == "CURRENT" && libjami::getParticipantList(aliceId, confId).size() == 3;
+        return aliceData_.conferenceChanged && carlaData_.hostState == "CURRENT" && libjami::getParticipantList(aliceId, confId).size() == 2;
     }));
 
     // get active calls = 1
@@ -437,7 +450,7 @@ ConversationCallTest::testRejoinCall()
     aliceData_.messages.clear();
     bobData_.messages.clear();
     carlaData_.messages.clear();
-    auto callId = libjami::placeCallWithMedia(aliceId, "swarm:" + aliceData_.id, {});
+    libjami::placeCallWithMedia(aliceId, "swarm:" + aliceData_.id, {});
     auto lastCommitIsCall = [&](const auto& data) {
         return !data.messages.empty()
                && data.messages.rbegin()->body.at("type") == "application/call-history+json";
@@ -456,7 +469,7 @@ ConversationCallTest::testRejoinCall()
                                    confId);
 
     aliceData_.conferenceChanged = false;
-    auto bobCall = libjami::placeCallWithMedia(bobId, destination, {});
+    libjami::placeCallWithMedia(bobId, destination, {});
     cv.wait_for(lk, 30s, [&]() {
         return aliceData_.conferenceChanged && bobData_.hostState == "CURRENT"  && bobData_.state == "CURRENT";
     });
@@ -466,18 +479,18 @@ ConversationCallTest::testRejoinCall()
         return aliceData_.conferenceChanged && carlaData_.hostState == "CURRENT" && carlaData_.state == "CURRENT";
     });
 
-    CPPUNIT_ASSERT(libjami::getParticipantList(aliceId, confId).size() == 3);
+    CPPUNIT_ASSERT(libjami::getParticipantList(aliceId, confId).size() == 2);
 
     // hangup 1 participant and rejoin
     aliceData_.messages.clear();
     bobData_.messages.clear();
     aliceData_.conferenceChanged = false;
-    Manager::instance().hangupCall(bobId, bobCall);
+    Manager::instance().hangupCall(bobId, bobData_.callId);
     cv.wait_for(lk, 30s, [&]() {
         return aliceData_.conferenceChanged && bobData_.hostState == "OVER";
     });
 
-    CPPUNIT_ASSERT(libjami::getParticipantList(aliceId, confId).size() == 2);
+    CPPUNIT_ASSERT(libjami::getParticipantList(aliceId, confId).size() == 1);
 
     aliceData_.conferenceChanged = false;
     libjami::placeCallWithMedia(bobId, destination, {});
@@ -485,7 +498,7 @@ ConversationCallTest::testRejoinCall()
         return aliceData_.conferenceChanged && bobData_.hostState == "CURRENT";
     });
 
-    CPPUNIT_ASSERT(libjami::getParticipantList(aliceId, confId).size() == 3);
+    CPPUNIT_ASSERT(libjami::getParticipantList(aliceId, confId).size() == 2);
     CPPUNIT_ASSERT(aliceData_.messages.empty());
     CPPUNIT_ASSERT(bobData_.messages.empty());
 
@@ -496,10 +509,10 @@ ConversationCallTest::testRejoinCall()
     Manager::instance().hangupConference(aliceId, confId);
 
     // should get message
-    cv.wait_for(lk, 30s, [&]() {
+    CPPUNIT_ASSERT(cv.wait_for(lk, 30s, [&]() {
         return !aliceData_.messages.empty() && !bobData_.messages.empty()
                && !carlaData_.messages.empty();
-    });
+    }));
     CPPUNIT_ASSERT(aliceData_.messages[0].body.find("duration") != aliceData_.messages[0].body.end());
     CPPUNIT_ASSERT(bobData_.messages[0].body.find("duration") != bobData_.messages[0].body.end());
     CPPUNIT_ASSERT(carlaData_.messages[0].body.find("duration") != carlaData_.messages[0].body.end());
@@ -533,7 +546,7 @@ ConversationCallTest::testParticipantHangupConfNotRemoved()
     // start call
     aliceData_.messages.clear();
     bobData_.messages.clear();
-    auto callId = libjami::placeCallWithMedia(aliceId, "swarm:" + aliceData_.id, {});
+    libjami::placeCallWithMedia(aliceId, "swarm:" + aliceData_.id, {});
     auto lastCommitIsCall = [&](const auto& data) {
         return !data.messages.empty()
                && data.messages.rbegin()->body.at("type") == "application/call-history+json";
@@ -597,7 +610,7 @@ ConversationCallTest::testJoinFinishedCall()
     aliceData_.messages.clear();
     bobData_.messages.clear();
     carlaData_.messages.clear();
-    auto callId = libjami::placeCallWithMedia(aliceId, "swarm:" + aliceData_.id, {});
+    libjami::placeCallWithMedia(aliceId, "swarm:" + aliceData_.id, {});
     auto lastCommitIsCall = [&](const auto& data) {
         return !data.messages.empty()
                && data.messages.rbegin()->body.at("type") == "application/call-history+json";
@@ -617,7 +630,7 @@ ConversationCallTest::testJoinFinishedCall()
     aliceData_.messages.clear();
     bobData_.messages.clear();
     carlaData_.messages.clear();
-    Manager::instance().hangupCall(aliceId, callId);
+    Manager::instance().hangupConference(aliceId, aliceData_.confId);
     // should get message
     cv.wait_for(lk, 30s, [&]() {
         return !aliceData_.messages.empty() && !bobData_.messages.empty()
@@ -694,7 +707,7 @@ ConversationCallTest::testJoinFinishedCallForbidden()
     aliceData_.messages.clear();
     bobData_.messages.clear();
     carlaData_.messages.clear();
-    auto callId = libjami::placeCallWithMedia(aliceId, "swarm:" + aliceData_.id, {});
+    libjami::placeCallWithMedia(aliceId, "swarm:" + aliceData_.id, {});
     auto lastCommitIsCall = [&](const auto& data) {
         return !data.messages.empty()
                && data.messages.rbegin()->body.at("type") == "application/call-history+json";
@@ -716,7 +729,7 @@ ConversationCallTest::testJoinFinishedCallForbidden()
     aliceData_.messages.clear();
     bobData_.messages.clear();
     carlaData_.messages.clear();
-    Manager::instance().hangupCall(aliceId, callId);
+    Manager::instance().hangupConference(aliceId, aliceData_.confId);
 
     // should get message
     cv.wait_for(lk, 30s, [&]() {
@@ -803,7 +816,7 @@ ConversationCallTest::testUsePreference()
     // start call
     aliceData_.messages.clear();
     bobData_.messages.clear();
-    auto callId = libjami::placeCallWithMedia(bobId, "swarm:" + aliceData_.id, {});
+    libjami::placeCallWithMedia(bobId, "swarm:" + aliceData_.id, {});
     auto lastCommitIsCall = [&](const auto& data) {
         return !data.messages.empty()
                && data.messages.rbegin()->body.at("type") == "application/call-history+json";
@@ -816,7 +829,11 @@ ConversationCallTest::testUsePreference()
 
     // Alice should be the host
     CPPUNIT_ASSERT(aliceAccount->getConference(confId));
-    Manager::instance().hangupCall(bobId, callId);
+    // Bob should be the only participant
+    CPPUNIT_ASSERT(cv.wait_for(lk, 5s, [&]() { return pInfos_.size() == 1; }));
+    auto uri = string_remove_suffix(pInfos_[0]["uri"], '@');
+    CPPUNIT_ASSERT(uri == bobUri);
+    Manager::instance().hangupConference(bobId, bobData_.confId);
 }
 
 void
@@ -834,7 +851,7 @@ ConversationCallTest::testJoinWhileActiveCall()
 
     // start call
     aliceData_.messages.clear();
-    auto callId = libjami::placeCallWithMedia(aliceId, "swarm:" + aliceData_.id, {});
+    libjami::placeCallWithMedia(aliceId, "swarm:" + aliceData_.id, {});
     auto lastCommitIsCall = [&](const auto& data) {
         return !data.messages.empty()
                && data.messages.rbegin()->body.at("type") == "application/call-history+json";
@@ -903,7 +920,7 @@ ConversationCallTest::testCallSelfIfDefaultHost()
     aliceData_.messages.clear();
     bobData_.messages.clear();
     pInfos_.clear();
-    auto callId = libjami::placeCallWithMedia(aliceId, "swarm:" + aliceData_.id, {});
+    libjami::placeCallWithMedia(aliceId, "swarm:" + aliceData_.id, {});
     auto lastCommitIsCall = [&](const auto& data) {
         return !data.messages.empty()
                && data.messages.rbegin()->body.at("type") == "application/call-history+json";
@@ -960,7 +977,7 @@ ConversationCallTest::testNeedsHost()
     // Disable Host
     Manager::instance().sendRegister(aliceId, false);
     // start call
-    auto callId = libjami::placeCallWithMedia(bobId, "swarm:" + aliceData_.id, {});
+    libjami::placeCallWithMedia(bobId, "swarm:" + aliceData_.id, {});
     // Can fail after 30 seconds if it triggers a new ICE request (before No response from DHT)
     CPPUNIT_ASSERT(cv.wait_for(lk, 40s, [&]() { return bobData_.needsHost; }));
 }
@@ -993,7 +1010,7 @@ ConversationCallTest::testAudioOnly()
             {libjami::Media::MediaAttributeKey::LABEL, "audio_0"}};
     mediaList.emplace_back(mediaAttribute);
     pInfos_.clear();
-    auto callId = libjami::placeCallWithMedia(aliceId, "swarm:" + aliceData_.id, mediaList);
+    libjami::placeCallWithMedia(aliceId, "swarm:" + aliceData_.id, mediaList);
     // should get message
     cv.wait_for(lk, 30s, [&]() { return !aliceData_.messages.empty() && !pInfos_.empty(); });
     CPPUNIT_ASSERT(aliceData_.messages[0].type == "application/call-history+json");
@@ -1002,7 +1019,7 @@ ConversationCallTest::testAudioOnly()
 
     // hangup
     aliceData_.messages.clear();
-    Manager::instance().hangupCall(aliceId, callId);
+    Manager::instance().hangupConference(aliceId, aliceData_.confId);
 
     // should get message
     cv.wait_for(lk, 30s, [&]() { return !aliceData_.messages.empty(); });
@@ -1049,7 +1066,7 @@ ConversationCallTest::testJoinAfterMuteHost()
             {libjami::Media::MediaAttributeKey::SOURCE, ""},
             {libjami::Media::MediaAttributeKey::LABEL, "audio_0"}};
     mediaList.emplace_back(mediaAttribute);
-    auto callId = libjami::placeCallWithMedia(aliceId, "swarm:" + aliceData_.id, mediaList);
+    libjami::placeCallWithMedia(aliceId, "swarm:" + aliceData_.id, mediaList);
     auto lastCommitIsCall = [&](const auto& data) {
         return !data.messages.empty()
                && data.messages.rbegin()->type == "application/call-history+json";
@@ -1061,8 +1078,8 @@ ConversationCallTest::testJoinAfterMuteHost()
     auto confId = bobData_.messages.rbegin()->body.at("confId");
 
     // Mute host
-    auto call = std::dynamic_pointer_cast<SIPCall>(aliceAccount->getCall(callId));
-    auto proposedList = call->currentMediaList();
+    auto conference = aliceAccount->getConference(aliceData_.confId);
+    auto proposedList = conference->currentMediaList();
     for (auto& media : proposedList)
         if (media["LABEL"] == "audio_0")
             media["MUTED"] = "true";
@@ -1086,7 +1103,135 @@ ConversationCallTest::testJoinAfterMuteHost()
 
 }
 
+void
+ConversationCallTest::testBusy()
+{
+    connectSignals();
+
+    auto aliceAccount = Manager::instance().getAccount<JamiAccount>(aliceId);
+    auto aliceUri = aliceAccount->getUsername();
+    auto bobAccount = Manager::instance().getAccount<JamiAccount>(bobId);
+    auto bobUri = bobAccount->getUsername();
+
+    aliceAccount->addContact(bobUri);
+    aliceAccount->sendTrustRequest(bobUri, {});
+    CPPUNIT_ASSERT(cv.wait_for(lk, 30s, [&]() { return bobData_.requestReceived; }));
+
+    CPPUNIT_ASSERT(bobAccount->acceptTrustRequest(aliceUri));
+    CPPUNIT_ASSERT(cv.wait_for(lk, 30s, [&]() { return !bobData_.id.empty(); }));
+
+
+    // start call
+    aliceData_.messages.clear();
+    bobData_.messages.clear();
+    carlaData_.messages.clear();
+    std::vector<std::map<std::string, std::string>> mediaList;
+    std::map<std::string, std::string> mediaAttribute
+        = {{libjami::Media::MediaAttributeKey::MEDIA_TYPE,
+            libjami::Media::MediaAttributeValue::AUDIO},
+            {libjami::Media::MediaAttributeKey::ENABLED, TRUE_STR},
+            {libjami::Media::MediaAttributeKey::MUTED, FALSE_STR},
+            {libjami::Media::MediaAttributeKey::SOURCE, ""},
+            {libjami::Media::MediaAttributeKey::LABEL, "audio_0"}};
+    mediaList.emplace_back(mediaAttribute);
+    libjami::placeCallWithMedia(aliceId, bobUri, mediaList);
+    auto lastCommitIsCall = [&](const auto& data) {
+        return !data.messages.empty()
+               && data.messages.rbegin()->type == "application/call-history+json";
+    };
+    // should get message
+    CPPUNIT_ASSERT(cv.wait_for(lk, 60s, [&]() {
+        return lastCommitIsCall(aliceData_);
+    }));
+    auto reason = aliceData_.messages.rbegin()->body.at("reason");
+    CPPUNIT_ASSERT(reason == "busy");
+}
+void
+ConversationCallTest::testDecline()
+{
+    connectSignals();
+
+    auto aliceAccount = Manager::instance().getAccount<JamiAccount>(aliceId);
+    auto aliceUri = aliceAccount->getUsername();
+    auto bobAccount = Manager::instance().getAccount<JamiAccount>(bobId);
+    auto bobUri = bobAccount->getUsername();
+
+    aliceAccount->addContact(bobUri);
+    aliceAccount->sendTrustRequest(bobUri, {});
+    CPPUNIT_ASSERT(cv.wait_for(lk, 30s, [&]() { return bobData_.requestReceived; }));
+
+    CPPUNIT_ASSERT(bobAccount->acceptTrustRequest(aliceUri));
+    CPPUNIT_ASSERT(cv.wait_for(lk, 30s, [&]() { return !bobData_.id.empty(); }));
+
+    // start call
+    aliceData_.messages.clear();
+    bobData_.messages.clear();
+    carlaData_.messages.clear();
+    std::vector<std::map<std::string, std::string>> mediaList;
+    std::map<std::string, std::string> mediaAttribute
+        = {{libjami::Media::MediaAttributeKey::MEDIA_TYPE,
+            libjami::Media::MediaAttributeValue::AUDIO},
+            {libjami::Media::MediaAttributeKey::ENABLED, TRUE_STR},
+            {libjami::Media::MediaAttributeKey::MUTED, FALSE_STR},
+            {libjami::Media::MediaAttributeKey::SOURCE, ""},
+            {libjami::Media::MediaAttributeKey::LABEL, "audio_0"}};
+    mediaList.emplace_back(mediaAttribute);
+    libjami::placeCallWithMedia(aliceId, bobUri, mediaList);
+    CPPUNIT_ASSERT(cv.wait_for(lk, 10s, [&]() { return !bobData_.callId.empty() && bobData_.state == "INCOMING"; }));
+
+    libjami::refuse(bobId, bobData_.callId);
+
+    auto lastCommitIsCall = [&](const auto& data) {
+        return !data.messages.empty()
+               && data.messages.rbegin()->type == "application/call-history+json";
+    };
+    // should get message
+    CPPUNIT_ASSERT(cv.wait_for(lk, 60s, [&]() {
+        return lastCommitIsCall(aliceData_);
+    }));
+    auto reason = aliceData_.messages.rbegin()->body.at("reason");
+    CPPUNIT_ASSERT(reason == "declined");
+}
+
+void
+ConversationCallTest::testNoDevice()
+{
+    connectSignals();
+
+    auto aliceAccount = Manager::instance().getAccount<JamiAccount>(aliceId);
+    auto aliceUri = aliceAccount->getUsername();
+
+    aliceAccount->addContact("e2eb225c76be68713d4874d290200849436c6355");
+    aliceAccount->sendTrustRequest("e2eb225c76be68713d4874d290200849436c6355", {});
+
+    // start call
+    aliceData_.messages.clear();
+    bobData_.messages.clear();
+    carlaData_.messages.clear();
+    std::vector<std::map<std::string, std::string>> mediaList;
+    std::map<std::string, std::string> mediaAttribute
+        = {{libjami::Media::MediaAttributeKey::MEDIA_TYPE,
+            libjami::Media::MediaAttributeValue::AUDIO},
+            {libjami::Media::MediaAttributeKey::ENABLED, TRUE_STR},
+            {libjami::Media::MediaAttributeKey::MUTED, FALSE_STR},
+            {libjami::Media::MediaAttributeKey::SOURCE, ""},
+            {libjami::Media::MediaAttributeKey::LABEL, "audio_0"}};
+    mediaList.emplace_back(mediaAttribute);
+    libjami::placeCallWithMedia(aliceId, "e2eb225c76be68713d4874d290200849436c6355", mediaList);
+
+    auto lastCommitIsCall = [&](const auto& data) {
+        return !data.messages.empty()
+               && data.messages.rbegin()->type == "application/call-history+json";
+    };
+    // should get message
+    CPPUNIT_ASSERT(cv.wait_for(lk, 60s, [&]() {
+        return lastCommitIsCall(aliceData_);
+    }));
+    auto reason = aliceData_.messages.rbegin()->body.at("reason");
+    CPPUNIT_ASSERT(reason == "no_device");
+}
+
 } // namespace test
 } // namespace jami
 
-RING_TEST_RUNNER(jami::test::ConversationCallTest::name())
+CORE_TEST_RUNNER(jami::test::ConversationCallTest::name())
