@@ -1,21 +1,18 @@
 /*
- *  Copyright (C) 2021-2023 Savoir-faire Linux Inc.
+ *  Copyright (C) 2004-2025 Savoir-faire Linux Inc.
  *
- *  Author: Sébastien Blin <sebastien.blin@savoirfairelinux.com>
- *
- *  This program is free software; you can redistribute it and/or modify
+ *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 3 of the License, or
+ *  the Free Software Foundation, either version 3 of the License, or
  *  (at your option) any later version.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA.
+ *  along with this program. If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include <cppunit/TestAssert.h>
@@ -36,6 +33,7 @@
 #include "jamidht/conversation.h"
 #include "jamidht/conversationrepository.h"
 #include "conversation/conversationcommon.h"
+#include "json_utils.h"
 
 using namespace std::string_literals;
 
@@ -51,6 +49,7 @@ addVote(std::shared_ptr<JamiAccount> account,
         const std::string& votedUri,
         const std::string& content)
 {
+    ConversationRepository::DISABLE_RESET = true;
     auto repoPath = fileutils::get_data_dir() / account->getAccountID()
                     / "conversations" / convId;
     auto voteDirectory = repoPath / "votes" / "members";
@@ -68,11 +67,8 @@ addVote(std::shared_ptr<JamiAccount> account,
     Json::Value json;
     json["uri"] = votedUri;
     json["type"] = "vote";
-    Json::StreamWriterBuilder wbuilder;
-    wbuilder["commentStyle"] = "None";
-    wbuilder["indentation"] = "";
-    ConversationRepository cr(account->weak(), convId);
-    cr.commitMessage(Json::writeString(wbuilder, json), false);
+    ConversationRepository cr(account, convId);
+    cr.commitMessage(json::toString(json), false);
 }
 
 void
@@ -80,6 +76,7 @@ simulateRemoval(std::shared_ptr<JamiAccount> account,
                 const std::string& convId,
                 const std::string& votedUri)
 {
+    ConversationRepository::DISABLE_RESET = true;
     auto repoPath = fileutils::get_data_dir() / account->getAccountID()
                     / "conversations" / convId;
     auto memberFile = repoPath / "members" / (votedUri + ".crt");
@@ -102,16 +99,13 @@ simulateRemoval(std::shared_ptr<JamiAccount> account,
     git_index_write(index.get());
     git_strarray_dispose(&array);
 
-    ConversationRepository cr(account->weak(), convId);
+    ConversationRepository cr(account, convId);
 
     Json::Value json;
     json["action"] = "ban";
     json["uri"] = votedUri;
     json["type"] = "member";
-    Json::StreamWriterBuilder wbuilder;
-    wbuilder["commentStyle"] = "None";
-    wbuilder["indentation"] = "";
-    cr.commitMessage(Json::writeString(wbuilder, json));
+    cr.commitMessage(json::toString(json));
 
     libjami::sendMessage(account->getAccountID(),
                        convId,
@@ -125,6 +119,7 @@ addFile(std::shared_ptr<JamiAccount> account,
         const std::string& relativePath,
         const std::string& content)
 {
+    ConversationRepository::DISABLE_RESET = true;
     auto repoPath = fileutils::get_data_dir() / account->getAccountID()
                     / "conversations" / convId;
     // Add file
@@ -155,6 +150,7 @@ addFile(std::shared_ptr<JamiAccount> account,
 void
 addAll(std::shared_ptr<JamiAccount> account, const std::string& convId)
 {
+    ConversationRepository::DISABLE_RESET = true;
     auto repoPath = fileutils::get_data_dir() / account->getAccountID()
                     / "conversations" / convId;
 
@@ -177,17 +173,15 @@ addAll(std::shared_ptr<JamiAccount> account, const std::string& convId)
 void
 commit(std::shared_ptr<JamiAccount> account, const std::string& convId, Json::Value& message)
 {
-    ConversationRepository cr(account->weak(), convId);
-
-    Json::StreamWriterBuilder wbuilder;
-    wbuilder["commentStyle"] = "None";
-    wbuilder["indentation"] = "";
-    cr.commitMessage(Json::writeString(wbuilder, message));
+    ConversationRepository::DISABLE_RESET = true;
+    ConversationRepository cr(account, convId);
+    cr.commitMessage(json::toString(message));
 }
 
 std::string
 commitInRepo(const std::string& path, std::shared_ptr<JamiAccount> account, const std::string& msg)
 {
+    ConversationRepository::DISABLE_RESET = true;
     auto deviceId = std::string(account->currentDeviceId());
     auto name = account->getDisplayName();
     if (name.empty())
@@ -206,12 +200,12 @@ commitInRepo(const std::string& path, std::shared_ptr<JamiAccount> account, cons
     git_repository* repo = nullptr;
     // TODO share this repo with GitServer
     if (git_repository_open(&repo, path.c_str()) != 0) {
-        JAMI_ERROR("Could not open repository");
+        JAMI_ERROR("Unable to open repository");
         return {};
     }
 
     if (git_repository_index(&index_ptr, repo) < 0) {
-        JAMI_ERROR("Could not open repository index");
+        JAMI_ERROR("Unable to open repository index");
         return {};
     }
     GitIndex index {index_ptr, git_index_free};
@@ -224,30 +218,35 @@ commitInRepo(const std::string& path, std::shared_ptr<JamiAccount> account, cons
 
     git_tree* tree_ptr = nullptr;
     if (git_tree_lookup(&tree_ptr, repo, &tree_id) < 0) {
-        JAMI_ERROR("Could not look up initial tree");
+        JAMI_ERROR("Unable to look up initial tree");
         return {};
     }
     GitTree tree = {tree_ptr, git_tree_free};
 
     git_oid commit_id;
     if (git_reference_name_to_id(&commit_id, repo, "HEAD") < 0) {
-        JAMI_ERROR("Cannot get reference for HEAD");
+        JAMI_ERROR("Unable to get reference for HEAD");
         return {};
     }
 
     git_commit* head_ptr = nullptr;
     if (git_commit_lookup(&head_ptr, repo, &commit_id) < 0) {
-        JAMI_ERROR("Could not look up HEAD commit");
+        JAMI_ERROR("Unable to look up HEAD commit");
         return {};
     }
     GitCommit head_commit {head_ptr, git_commit_free};
 
     git_buf to_sign = {};
+#if LIBGIT2_VER_MAJOR == 1 && LIBGIT2_VER_MINOR == 8 && \
+    (LIBGIT2_VER_REVISION == 0 || LIBGIT2_VER_REVISION == 1 || LIBGIT2_VER_REVISION == 3)
+    git_commit* const head_ref[1] = {head_commit.get()};
+#else
     const git_commit* head_ref[1] = {head_commit.get()};
+#endif
     if (git_commit_create_buffer(
             &to_sign, repo, sig.get(), sig.get(), nullptr, msg.c_str(), tree.get(), 1, &head_ref[0])
         < 0) {
-        JAMI_ERROR("Could not create commit buffer");
+        JAMI_ERROR("Unable to create commit buffer");
         return {};
     }
 
@@ -263,7 +262,7 @@ commitInRepo(const std::string& path, std::shared_ptr<JamiAccount> account, cons
         < 0) {
         const git_error* err = giterr_last();
         if (err)
-            JAMI_ERROR("Could not sign commit: {}", err->message);
+            JAMI_ERROR("Unable to sign commit: {}", err->message);
         return {};
     }
 
@@ -272,7 +271,7 @@ commitInRepo(const std::string& path, std::shared_ptr<JamiAccount> account, cons
     if (git_reference_create(&ref_ptr, repo, "refs/heads/main", &commit_id, true, nullptr) < 0) {
         const git_error* err = giterr_last();
         if (err)
-            JAMI_ERROR("Could not move commit to main: {}", err->message);
+            JAMI_ERROR("Unable to move commit to main: {}", err->message);
         return {};
     }
     git_reference_free(ref_ptr);
