@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2014-2022 Savoir-faire Linux Inc.
+ *  Copyright (C) 2014-2025 Savoir-faire Linux Inc.
  *  Author(s) : Adrien Béraud <adrien.beraud@savoirfairelinux.com>
  *              Simon Désaulniers <simon.desaulniers@savoirfairelinux.com>
  *              Nicolas Reynaud <nicolas.reynaud@savoirfairelinux.com>
@@ -316,7 +316,7 @@ void Pht::updateCanary(Prefix p) {
     dht_->put(p.hash(), std::move(canary_value),
         [=](bool){
             static std::bernoulli_distribution d(0.5);
-            crypto::random_device rd;
+            std::random_device rd;
             if (p.size_ and d(rd))
                 updateCanary(p.getPrefix(-1));
         }
