@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2014-2023 Savoir-faire Linux Inc.
+ *  Copyright (C) 2014-2025 Savoir-faire Linux Inc.
  *  Author(s) : Adrien Béraud <adrien.beraud@savoirfairelinux.com>
  *              Simon Désaulniers <simon.desaulniers@savoirfairelinux.com>
  *
@@ -73,7 +73,7 @@ std::ostream& operator<< (std::ostream& s, const Value& v)
                 s << "data(" << v.user_type << "):";
             if (v.user_type == "text/plain") {
                 s << '"';
-                s.write((const char*)v.data.data(), v.data.size());
+                s.write(reinterpret_cast<const char*>(v.data.data()), v.data.size());
                 s << '"';
             } else if (v.data.size() < 1024) {
                 s << toHex(v.data.data(), v.data.size());
@@ -271,6 +271,7 @@ Value::encrypt(const crypto::PrivateKey& from, const crypto::PublicKey& to)
     setRecipient(to.getId());
     sign(from);
     Value nv {id};
+    nv.priority = priority;
     nv.setCypher(to.encrypt(getToEncrypt()));
     return nv;
 }
